# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
from __future__ import print_function
from collections import defaultdict
import logging

# validate, , validate things, internal
from yotta.lib import validate

def addOptions(parser):
    parser.add_argument('--all', '-a', dest='list_all', default=False, action='store_true',
        help='List all licenses, not just each unique license.'
    )

def execCommand(args, following_args):
    c = validate.currentDirectoryModule()
    if not c:
        return 1

    if not args.target:
        logging.error('No target has been set, use "yotta target" to set one.')
        return 1

    target, errors = c.satisfyTarget(args.target)
    if errors:
        for error in errors:
            logging.error(error)
        return 1

    dependencies = c.getDependenciesRecursive(
                      target = target,
        available_components = [(c.getName(), c)]
    )

    errors = []
    if args.list_all:
        for name, dep in dependencies.items():
            if not dep:
                errors.append(u'%s is missing: license unknown!' % name)
            else:
                print(u'%s: %s' % (name, u', '.join(dep.licenses())))
    else:
        licenses = defaultdict(list)
        for name, dep in dependencies.items():
            if not dep:
                errors.append(u'%s is missing: license unknown!' % name)
            else:
                for lic in dep.licenses():
                    licenses[lic].append(name)
        for lic in licenses:
            print(lic)

    if len(errors):
        for err in errors:
            logging.error(err)
        return 1
    return 0




# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import argparse
import logging
import os

# version, , represent versions and specifications, internal
from yotta.lib import version
# Component, , represents an installed component, internal
from yotta.lib import component
# Target, , represents an installed target, internal
from yotta.lib import target
# vcs, , represent version controlled directories, internal
from yotta.lib import vcs

def addOptions(parser):
    def patchType(s):
        if s.lower() in ('major', 'minor', 'patch'):
            return s.lower()
        try:
            return version.Version(s)
        except:
            raise argparse.ArgumentTypeError(
                '"%s" is not a valid version (expected patch, major, minor, or something like 1.2.3)' % s
            )
    parser.add_argument('action', type=patchType, nargs='?', help='[patch | minor | major | <version>]')


def execCommand(args, following_args):
    wd = os.getcwd()
    c = component.Component(wd)
    # skip testing for target if we already found a component
    t = None if c else target.Target(wd)
    if not (c or t):
        logging.debug(str(c.getError()))
        if t:
            logging.debug(str(t.getError()))
        logging.error('The current directory does not contain a valid module or target.')
        return 1
    else:
        # only needed separate objects in order to display errors
        p = (c or t)

    if args.action:
        try:
            if not p.vcsIsClean():
                logging.error('The working directory is not clean')
                return 1

            v = p.getVersion()
            pre_script_env = {
                'YOTTA_OLD_VERSION':str(v)
            }
            if args.action in ('major', 'minor', 'patch'):
                v.bump(args.action)
            else:
                v = args.action

            pre_script_env['YOTTA_NEW_VERSION'] =str(v)
            errcode = p.runScript('preVersion', pre_script_env)
            if errcode:
                return errcode

            logging.info('@%s' % v)
            p.setVersion(v)

            p.writeDescription()

            errcode = p.runScript('postVersion')
            if errcode:
                return errcode

            p.commitVCS(tag='v'+str(v))
        except vcs.VCSError as e:
            logging.error(e)
    else:
        logging.info(str(p.getVersion()))




# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import os
import logging

# Access common, , components shared between access modules, internal
from yotta.lib import access_common
# Registry Access, , access packages in the registry, internal
from yotta.lib import registry_access
# Github Access, , access repositories on github, internal
from yotta.lib import github_access
# Git Access, , access repositories via generic git URLs, internal
from yotta.lib import git_access
# hg Access, , access repositories via generic mercurial URLs, internal
from yotta.lib import hg_access
# fsutils, , misc filesystem utils, internal
from yotta.lib import fsutils
# sourceparse, , parse version source urls, internal
from yotta.lib import sourceparse

# Version requirement strings we want to support:
#
# (>,>=,<,<=,)version                         # central repo
# 1.2.x                                       # central repo
# http://...                                  # tarball or zipfile
# owner/repo @((>,>=,<,<=,,)version)          # Github
# hg+(ssh://..., http://...)(#hash-or-tag)    # hg
# * (any version)                             # central repo
#
# Currently should work:
# *
# <,>,>= etc
# 1.2.x
# owner/repo
# git://github.com/user/project(#version)     # Github
# git+(ssh://..., http://...)(#hash-or-tag)   # git
# anything://anything.git                     # git
#


logger = logging.getLogger('access')


def remoteComponentFor(name, version_required, registry='modules'):
    ''' Return a RemoteComponent sublclass for the specified component name and
        source url (or version specification)
        Raises an exception if any arguments are invalid.
    '''

    try:
        vs = sourceparse.parseSourceURL(version_required)
    except ValueError as e:
        raise access_common.Unavailable(
            '%s' % (e)
        )

    if vs.source_type == 'registry':
        if registry not in ('modules', 'targets'):
            raise Exception('no known registry namespace "%s"' % registry)
        return registry_access.RegistryThing.createFromSource(
            vs, name, registry=registry
        )
    elif vs.source_type == 'github':
        return github_access.GithubComponent.createFromSource(vs, name)
    elif vs.source_type == 'git':
        return git_access.GitComponent.createFromSource(vs, name)
    elif vs.source_type == 'hg':
        return hg_access.HGComponent.createFromSource(vs, name)
    else:
        raise Exception('unsupported module source: "%s"' % vs.source_type)
    # !!! FIXME: next: generic http urls to tarballs


def tagOrBranchVersion(spec, tags, branches, diagnostic_name):
    for i, v in enumerate(tags + branches):
        if spec == v.tag:
            if i >= len(tags):
                logger.warning(
                    'Using head of "%s" branch for "%s", not a tagged version' % (
                        v.tag,
                        diagnostic_name
                    )
                )
            return v
    return None

def latestSuitableVersion(name, version_required, registry='modules', quiet=False):
    ''' Return a RemoteVersion object representing the latest suitable
        version of the named component or target.

        All RemoteVersion objects have a .unpackInto(directory) method.
    '''

    remote_component = remoteComponentFor(name, version_required, registry)

    if quiet:
        logger.debug('get versions for ' + name)
    else:
        logger.info('get versions for ' + name)

    if remote_component.remoteType() == 'registry':
        logger.debug('satisfy %s from %s registry' % (name, registry))
        vers = remote_component.availableVersions()
        spec = remote_component.versionSpec()
        v = spec.select(vers)
        logger.debug("%s selected %s from %s", spec, v, vers)
        if not v:
            raise access_common.Unavailable(
                'The %s registry does not provide a version of "%s" matching "%s"' % (
                    registry, name, spec
                )
            )
        return v
    elif remote_component.remoteType() == 'github':
        logger.debug('satisfy %s from github url' % name)
        spec = remote_component.versionSpec()
        if spec:
            vers = remote_component.availableVersions()
            if not len(vers):
                logger.warning(
                    'Github repository "%s" has no tagged versions, default branch will be used' % (
                        remote_component.repo
                    )
                )
                vers = [remote_component.tipVersion()]
            v = spec.select(vers)
            logger.debug("%s selected %s from %s", spec, v, vers)
            if not v:
                raise access_common.Unavailable(
                    'Github repository "%s" does not provide a version matching "%s"' % (
                        remote_component.repo,
                        remote_component.spec
                    )
                )
            return v
        else:
            # we're fetching a specific tag, or the head of a branch:
            v = tagOrBranchVersion(
                remote_component.tagOrBranchSpec(),
                remote_component.availableTags(),
                remote_component.availableBranches(),
                name
            )
            if v:
                return v
            raise access_common.Unavailable(
                'Github repository "%s" does not have any tags or branches matching "%s"' % (
                    version_required, remote_component.tagOrBranchSpec()
                )
            )

    elif remote_component.remoteType() in ('git', 'hg'):
        clone_type = remote_component.remoteType()
        logger.debug('satisfy %s from %s url' % (name, clone_type))
        local_clone = remote_component.clone()
        if not local_clone:
            raise access_common.Unavailable(
                'Failed to clone %s URL %s to satisfy dependency %s' % (clone_type, version_required, name)
            )
        spec = remote_component.versionSpec()
        if spec:
            vers = local_clone.availableVersions()
            if not len(vers):
                logger.warning(
                    '%s repository "%s" has no tagged versions, default branch will be used' % (clone_type, version_required)
                )
                vers = [local_clone.tipVersion()]
            v = spec.select(vers)
            logger.debug("%s selected %s from %s", spec, v, vers)
            if not v:
                raise access_common.Unavailable(
                    '%s repository "%s" does not provide a version matching "%s"' % (
                        clone_type,
                        version_required,
                        remote_component.spec
                    )
                )
            return v
        elif remote_component.remoteType() == 'git':
            v = tagOrBranchVersion(
                remote_component.tagOrBranchSpec(),
                local_clone.availableTags(),
                local_clone.availableBranches(),
                name
            )
            if v:
                return v
            raise access_common.Unavailable(
                '%s repository "%s" does not have any tags or branches matching "%s"' % (
                    clone_type, version_required, spec
                )
            )
        else:
            raise Exception("invalid spec for hg source: tags/branches are not supported yet!")

    # !!! FIXME: next: generic http urls to tarballs

    return None

def searchPathsFor(name, spec, search_paths, type='module', inherit_shrinkwrap=None):
    for path in search_paths:
        check_path = os.path.join(path, name)
        logger.debug("check path %s for %s" % (check_path, name))
        instance = _clsForType(type)(
                     check_path,
                   installed_linked = fsutils.isLink(check_path),
            latest_suitable_version = None,
                 inherit_shrinkwrap = inherit_shrinkwrap
        )
        if instance:
            logger.debug("got %s v=%s spec %s matches? %s", instance, instance.getVersion(), spec, spec.match(instance.getVersion()))
            if spec.match(instance.getVersion()):
                return instance
        else:
            logger.debug("got %s", instance)
    return None

def _registryNamespaceForType(type):
    assert(type in ('module', 'target'))
    return type + 's'

def _clsForType(type):
    # Component, , represents an installed component, internal
    from yotta.lib import component
    # Target, , represents an installed target, internal
    from yotta.lib import target
    assert(type in ('module', 'target'))
    return {'module':component.Component, 'target':target.Target}[type]

def satisfyFromAvailable(name, available, type='module'):
    # we don't need to pass the shrinkwrap to this function because if this
    # function finds a module we have already processed its dependencies for
    # another module - so the shrinkwrap could not be applied.
    # !!! FIXME: what about issuing warnings for things that don't match the
    # shrinkwrap though?
    if name in available and available[name]:
        logger.debug('satisfy %s from already installed %ss' % (name, type))
        r = available[name]
        if name != r.getName():
            raise access_common.Unavailable('%s %s was installed as different name %s in %s' % (
                type, r.getName(), name, r.path
            ))
        return r
    return None

def satisfyVersionFromSearchPaths(name, version_required, search_paths, update=False, type='module', inherit_shrinkwrap=None):
    ''' returns a Component/Target for the specified version, if found in the
        list of search paths. If `update' is True, then also check for newer
        versions of the found component, and update it in-place (unless it was
        installed via a symlink).
    '''
    # Pack, , base class for targets and components, internal
    from yotta.lib import pack

    v = None
    try:
        sv = sourceparse.parseSourceURL(version_required)
    except ValueError as e:
        logging.error(e)
        return None

    try:
        local_version = searchPathsFor(
            name,
            sv.semanticSpec(),
            search_paths,
            type,
            inherit_shrinkwrap = inherit_shrinkwrap
        )
    except pack.InvalidDescription as e:
        logger.error(e)
        return None

    logger.debug("%s %s locally" % (('found', 'not found')[not local_version], name))
    if local_version:
        if update and not local_version.installedLinked():
            #logger.debug('attempt to check latest version of %s @%s...' % (name, version_required))
            v = latestSuitableVersion(name, version_required, registry=_registryNamespaceForType(type))
            if local_version:
                local_version.setLatestAvailable(v)

        # if we don't need to update, then we're done
        if local_version.installedLinked() or not local_version.outdated():
            logger.debug("satisfy component from directory: %s" % local_version.path)
            # if a component exists (has a valid description file), and either is
            # not outdated, or we are not updating
            if name != local_version.getName():
                raise Exception('Component %s found in incorrectly named directory %s (%s)' % (
                    local_version.getName(), name, local_version.path
                ))
            return local_version

        # otherwise, we need to update the installed component
        logger.info('update outdated: %s@%s -> %s' % (
            name,
            local_version.getVersion(),
            v
        ))
        # must rm the old component before continuing
        fsutils.rmRf(local_version.path)
        return _satisfyVersionByInstallingVersion(
            name, version_required, local_version.path, v, type=type, inherit_shrinkwrap=inherit_shrinkwrap
        )
    return None

def satisfyVersionByInstalling(name, version_required, working_directory, type='module', inherit_shrinkwrap=None):
    ''' installs and returns a Component/Target for the specified name+version
        requirement, into a subdirectory of `working_directory'
    '''
    v = latestSuitableVersion(name, version_required, _registryNamespaceForType(type))
    install_into = os.path.join(working_directory, name)
    return _satisfyVersionByInstallingVersion(
        name, version_required, install_into, v, type=type, inherit_shrinkwrap = inherit_shrinkwrap
    )

def _satisfyVersionByInstallingVersion(name, version_required, working_directory, version, type='module', inherit_shrinkwrap=None):
    ''' installs and returns a Component/Target for the specified version requirement into
        'working_directory' using the provided remote version object.
        This function is not normally called via `satisfyVersionByInstalling',
        which looks up a suitable remote version object.
    '''
    assert(version)
    logger.info('download %s', version)
    version.unpackInto(working_directory)
    r = _clsForType(type)(working_directory, inherit_shrinkwrap = inherit_shrinkwrap)
    if not r:
        raise Exception(
            'Dependency "%s":"%s" is not a valid %s.' % (name, version_required, type)
        )
    if name != r.getName():
        raise Exception('%s %s (specification %s) has incorrect name %s' % (
            type, name, version_required, r.getName()
        ))
    # error code deliberately ignored here for now, it isn't clear what the
    # behaviour should be (abort? remove the unpacked state then abort?
    # continue?)
    r.runScript('postInstall')
    return r

def satisfyVersion(
        name,
        version_required,
        available,
        search_paths,
        working_directory,
        update_installed=None,
        type='module',  # or 'target'
        inherit_shrinkwrap=None
    ):
    ''' returns a Component/Target for the specified version (either to an already
        installed copy (from the available list, or from disk), or to a newly
        downloaded one), or None if the version could not be satisfied.

        update_installed = None / 'Update'
            None:   prevent any attempt to look for new versions if the
                    component/target already exists
            Update: replace any existing version with the newest available, if
                    the newest available has a higher version
    '''

    r = satisfyFromAvailable(name, available, type=type)
    if r is not None:
        if not sourceparse.parseSourceURL(version_required).semanticSpecMatches(r.getVersion()):
            raise access_common.SpecificationNotMet(
                "Installed %s %s doesn't match specification %s" % (type, name, version_required)
            )
        return r

    r = satisfyVersionFromSearchPaths(
        name,
        version_required,
        search_paths,
        (update_installed == 'Update'),
        type = type,
        inherit_shrinkwrap = inherit_shrinkwrap
    )
    if r is not None:
        return r

    return satisfyVersionByInstalling(
        name, version_required, working_directory, type=type, inherit_shrinkwrap = inherit_shrinkwrap
    )


def satisfyTarget(name, version_required, working_directory, update_installed=None):
    return satisfyVersion(
                    name = name,
        version_required = version_required,
               available = {},
            search_paths = [working_directory],
       working_directory = working_directory,
        update_installed = update_installed,
                    type = 'target',
      inherit_shrinkwrap = inherit_shrinkwrap
    )



#!/usr/bin/env python
# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest

# sourceparse, , parse version source urls, internal
from yotta.lib import sourceparse


Registry_URLs = [
    '',
    '*',
    '1.2.3',
    '>=1.2.3',
    '^0.1.2',
    '~0.1.2',
]

Github_URLs = [
    'username/reponame',
    'username/reponame#1.2.3',
    'username/reponame#^1.2.3',
    'username/reponame#-1.2.3',
    'username/reponame#branch-or-tag-name',
    'username/reponame@1.2.3',
    'username/reponame@^1.2.3',
    'username/reponame@-1.2.3',
    'username/reponame@branch-or-tag-name',
]

Git_URLs = [
    'git+ssh://somewhere.com/something/etc/etc',
    'git+ssh://somewhere.com/something/etc/etc#1.2.3',
    'git+ssh://somewhere.com/something/etc/etc#^1.2.3',
    'git+ssh://somewhere.com/something/etc/etc#~1.2.3',
    'git+ssh://somewhere.com/something/etc/etc#branch-name',
    'ssh://somewhere.com/something/etc/etc.git',
    'ssh://somewhere.com/something/etc/etc.git#^1.2.3',
    'ssh://somewhere.com/something/etc/etc.git#~1.2.3',
    'ssh://somewhere.com/something/etc/etc.git#branch-name',
    'http://somewhere.something/something.git',
]

HG_URLs = [
    'hg+ssh://somewhere.com/something/etc/etc',
    'hg+ssh://somewhere.com/something/etc/etc#1.2.3',
    'hg+ssh://somewhere.com/something/etc/etc#^1.2.3',
    'hg+ssh://somewhere.com/something/etc/etc#~1.2.3',
    'ssh://somewhere.com/something/etc/etc.hg',
    'ssh://somewhere.com/something/etc/etc.hg#^1.2.3',
    'ssh://somewhere.com/something/etc/etc.hg#~1.2.3',
    'http://somewhere.something/something.hg',
]

test_invalid_urls = [
    'stuff otherstuff',
    'somthing/somethingelse/somethingelseagain'
]


class TestParseSourceURL(unittest.TestCase):
    def test_registryURLs(self):
        for url in Registry_URLs:
            sv = sourceparse.parseSourceURL(url)
            self.assertEqual(sv.source_type, 'registry')

    def test_githubURLs(self):
        for url in Github_URLs:
            sv = sourceparse.parseSourceURL(url)
            self.assertEqual(sv.source_type, 'github')

    def test_gitURLs(self):
        for url in Git_URLs:
            sv = sourceparse.parseSourceURL(url)
            self.assertEqual(sv.source_type, 'git')

    def test_hgURLs(self):
        for url in HG_URLs:
            sv = sourceparse.parseSourceURL(url)
            self.assertEqual(sv.source_type, 'hg')

    def test_invalid(self):
        for url in test_invalid_urls:
            self.assertRaises(ValueError, sourceparse.parseSourceURL, url)

Valid_Names = [
    'validname',
    'another-validname',
    'another-valid2name123'
]

class TestParseModuleNameAndSpec(unittest.TestCase):
    def test_validNames(self):
        for name in Valid_Names:
            n, s = sourceparse.parseModuleNameAndSpec(name)
            self.assertEqual(n, name)
            self.assertEqual(s, '*')

    def test_GithubRefs(self):
        for url in Github_URLs:
            n, s = sourceparse.parseModuleNameAndSpec(url)
            self.assertEqual(n, 'reponame')

    def test_atVersion(self):
        for name in Valid_Names:
            for v in Registry_URLs:
                if len(v):
                    nv = name + '@' + v
                    n, s = sourceparse.parseModuleNameAndSpec(nv)
                    self.assertEqual(n, name)
                    self.assertEqual(s, v)

    def test_atGitURL(self):
        for name in Valid_Names:
            for v in Git_URLs:
                nv = name + '@' + v
                n, s = sourceparse.parseModuleNameAndSpec(nv)
                self.assertEqual(n, name)
                self.assertEqual(s, v)

    def test_atHGURL(self):
        for name in Valid_Names:
            for v in HG_URLs:
                nv = name + '@' + v
                n, s = sourceparse.parseModuleNameAndSpec(nv)
                self.assertEqual(n, name)
                self.assertEqual(s, v)


if __name__ == '__main__':
    unittest.main()





#! /usr/bin/env python2.7
# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

import unittest
import logging
from collections import OrderedDict

from yotta.lib import target

logging.basicConfig(
    level=logging.ERROR
)

class ComponentTestCase(unittest.TestCase):
    def test_mergeDictionaries(self):
        a = OrderedDict([('a', 1), ('b', 2), ('c', 3), ('subdict', OrderedDict([('a',7), ('c',0)]))])
        b = OrderedDict([('a', 2), ('d', 4), ('e', 5), ('subdict', OrderedDict([('a',12), ('b',8), ('subsubdict', OrderedDict([(1,'a'), (2,'b')]))]))])
        c = OrderedDict([('subdict', {'subsubdict':{3:'c'}})])

        self.assertEqual(target._mergeDictionaries(a, {}), a)
        self.assertEqual(target._mergeDictionaries(b, {}), b)
        self.assertEqual(target._mergeDictionaries(c, {}), c)

        self.assertEqual(
            target._mergeDictionaries(a, b),
            OrderedDict([
                ('a', 1),
                ('b', 2),
                ('c', 3),
                ('subdict', OrderedDict([('a', 7), ('c', 0), ('b',8), ('subsubdict', OrderedDict([(1,'a'), (2,'b')]))])),
                ('d', 4),
                ('e', 5)
            ])
        )
        self.assertEqual(
            target._mergeDictionaries(target._mergeDictionaries(a, b), c),
            OrderedDict([
                ('a', 1),
                ('b', 2),
                ('c', 3),
                ('subdict', OrderedDict([('a', 7), ('c', 0), ('b',8), ('subsubdict', OrderedDict([(1,'a'), (2,'b'), (3,'c')]))])),
                ('d', 4),
                ('e', 5)
            ])
        )




#!/usr/bin/env python
# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest
import os
import tempfile
import re

# internal modules:
from yotta.lib.fsutils import rmRf
from yotta.test.cli import cli
from yotta.test.cli import util
from yotta.lib import settings


Test_Name = 'testing-dummy'
Test_Github_Name = "autopulated/github-access-testing"
Test_Target = "x86-osx-native,*"

Test_Module_JSON = '''{
  "name": "testmod",
  "version": "0.0.0",
  "description": "a test module",
  "keywords": [
  ],
  "author": "Someone Somewhere <someone.somewhere@yottabuild.org>",
  "repository": {
    "url": "git@github.com:mbedmicro/testmod.git",
    "type": "git"
  },
  "homepage": "https://github.com/mbedmicro/testmod",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "dependencies": {
    "testing-dummy": "*"
  },
  "targetDependencies": {
    "x86-osx-native": {
        "other-testing-dummy": "git@bitbucket.org:autopulated/other-testing-dummy.git#0.0.1"
    }
  },
  "testDependencies": {
    "test-testing-dummy": "~0.0.2"
  },
  "testTargetDependencies": {
    "x86-osx-native": {
      "test-target-dep": "~0.0.1"
    }
  }
}
'''

Test_Complex_Module_JSON = '''{
  "name": "test-testdep-a",
  "version": "0.0.2",
  "description": "Module to test test-dependencies",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "dependencies": {
    "test-testdep-b": "0.0.8",
    "test-testdep-c": "*",
    "test-testdep-d": "*"
  },
  "testDependencies": {
    "test-testdep-e": "*"
  }
}
'''

def hasGithubConfig():
    # can't run tests that hit github without an authn token
    if not settings.getProperty('github', 'authtoken'):
        return False
    return True

class TestCLIInstall(unittest.TestCase):
    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installRegistryRef(self):
        test_dir = tempfile.mkdtemp()
        stdout = self.runCheckCommand(['--target', Test_Target, 'install', Test_Name], test_dir)
        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installGithubRef(self):
        test_dir = tempfile.mkdtemp()
        stdout = self.runCheckCommand(['--target', Test_Target, 'install', Test_Github_Name], test_dir)
        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installDeps(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install'], test_dir)

        # also sanity-check listing:
        stdout = self.runCheckCommand(['--target', Test_Target, 'ls'], test_dir)
        self.assertIn('testmod', stdout)
        self.assertIn('other-testing-dummy', stdout)
        self.assertIn('test-testing-dummy', stdout)
        self.assertIn('test-target-dep', stdout)

        stdout = self.runCheckCommand(['--target', Test_Target, 'ls', '-a'], test_dir)
        self.assertIn('testmod', stdout)
        self.assertIn('other-testing-dummy', stdout)
        self.assertIn('test-testing-dummy', stdout)
        self.assertIn('test-target-dep', stdout)

        # and test install <modulename>
        stdout = self.runCheckCommand(['--target', Test_Target, 'install', 'hg-access-testing'], test_dir)
        stdout = self.runCheckCommand(['--target', Test_Target, 'ls'], test_dir)
        self.assertIn('hg-access-testing', stdout)
        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installAllTestDeps(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install', '--test-dependencies', 'all'], test_dir)

        # also sanity-check listing:
        stdout = self.runCheckCommand(['--target', Test_Target, 'ls', '-a'], test_dir)
        self.assertIn('testmod', stdout)
        self.assertIn('other-testing-dummy', stdout)
        self.assertIn('test-testing-dummy', stdout)
        self.assertIn('test-target-dep', stdout)
        self.assertNotIn('missing', stdout)
        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installNoTestDeps(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install', '--test-dependencies', 'none'], test_dir)

        # also sanity-check listing:
        stdout = self.runCheckCommand(['--target', Test_Target, 'ls'], test_dir)
        self.assertIn('testmod', stdout)
        self.assertIn('other-testing-dummy', stdout)
        self.assertTrue(re.search('test-testing-dummy.*missing', stdout))
        self.assertTrue(re.search('test-target-dep.*missing', stdout))

        stdout = self.runCheckCommand(['--target', Test_Target, 'ls', '-a'], test_dir)
        self.assertIn('testmod', stdout)
        self.assertIn('other-testing-dummy', stdout)
        self.assertTrue(re.search('test-testing-dummy.*missing', stdout))
        self.assertTrue(re.search('test-target-dep.*missing', stdout))

        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installComplexDeps(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Complex_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install'], test_dir)

        # also sanity-check listing:
        stdout = self.runCheckCommand(['--target', Test_Target, 'ls'], test_dir)

        self.assertIn('test-testdep-b', stdout)
        self.assertIn('test-testdep-c', stdout)
        self.assertIn('test-testdep-d', stdout)
        self.assertIn('test-testdep-e', stdout)
        self.assertIn('test-testdep-f', stdout)
        self.assertIn('test-testdep-h', stdout)
        self.assertNotIn('test-testdep-g', stdout)
        self.assertNotIn('test-testdep-i', stdout)
        self.assertNotIn('test-testdep-j', stdout)
        self.assertNotIn('test-testdep-k', stdout)
        self.assertNotIn('missing', stdout)

        stdout = self.runCheckCommand(['--target', Test_Target, 'ls', '-a'], test_dir)

        self.assertIn('test-testdep-b', stdout)
        self.assertIn('test-testdep-c', stdout)
        self.assertIn('test-testdep-d', stdout)
        self.assertIn('test-testdep-e', stdout)
        self.assertIn('test-testdep-f', stdout)
        self.assertIn('test-testdep-h', stdout)
        self.assertTrue(re.search('test-testdep-nodeps.*missing', stdout))
        self.assertTrue(re.search('test-testdep-i.*missing', stdout))
        self.assertTrue(re.search('test-testdep-g.*missing', stdout))
        self.assertNotIn('test-testdep-j', stdout)
        self.assertNotIn('test-testdep-k', stdout)

        # test update
        stdout = self.runCheckCommand(['--target', Test_Target, 'up'], test_dir)

        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installAllComplexTestDeps(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Complex_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install', '--test-dependencies', 'all'], test_dir)

        # also sanity-check listing:
        stdout = self.runCheckCommand(['--target', Test_Target, 'ls', '-a'], test_dir)

        self.assertIn('test-testdep-b', stdout)
        self.assertIn('test-testdep-c', stdout)
        self.assertIn('test-testdep-d', stdout)
        self.assertIn('test-testdep-e', stdout)
        self.assertIn('test-testdep-f', stdout)
        self.assertIn('test-testdep-g', stdout)
        self.assertIn('test-testdep-h', stdout)
        self.assertIn('test-testdep-i', stdout)
        self.assertNotIn('test-testdep-j', stdout)
        self.assertNotIn('test-testdep-k', stdout)
        self.assertNotIn('missing', stdout)

        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installNoComplexTestDeps(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Complex_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install', '--test-dependencies', 'none'], test_dir)

        # also sanity-check listing:
        stdout = self.runCheckCommand(['--target', Test_Target, 'ls'], test_dir)

        self.assertIn('test-testdep-b', stdout)
        self.assertIn('test-testdep-c', stdout)
        self.assertIn('test-testdep-d', stdout)
        # e should be installed because it is both a test dep and non-test dep:
        # maybe it shouldn't show up in the listing without -a though?
        self.assertIn('test-testdep-e', stdout)
        self.assertIn('test-testdep-f', stdout)
        self.assertIn('test-testdep-h', stdout)
        self.assertNotIn('test-testdep-g', stdout)
        self.assertNotIn('test-testdep-i', stdout)
        self.assertNotIn('test-testdep-j', stdout)
        self.assertNotIn('test-testdep-k', stdout)
        self.assertNotIn('missing', stdout)

        stdout = self.runCheckCommand(['--target', Test_Target, 'ls', '-a'], test_dir)

        self.assertIn('test-testdep-b', stdout)
        self.assertIn('test-testdep-c', stdout)
        self.assertIn('test-testdep-d', stdout)
        self.assertIn('test-testdep-e', stdout)
        self.assertIn('test-testdep-f', stdout)
        self.assertIn('test-testdep-h', stdout)
        self.assertTrue(re.search('test-testdep-nodeps.*missing', stdout))
        self.assertTrue(re.search('test-testdep-i.*missing', stdout))
        self.assertTrue(re.search('test-testdep-g.*missing', stdout))
        self.assertNotIn('test-testdep-j', stdout)
        self.assertNotIn('test-testdep-k', stdout)

        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_remove(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install'], test_dir)
        self.assertTrue(os.path.exists(os.path.join(test_dir, 'yotta_modules', 'testing-dummy')))

        self.runCheckCommand(['remove', 'testing-dummy'], test_dir)
        self.assertFalse(os.path.exists(os.path.join(test_dir, 'yotta_modules', 'testing-dummy')))

        stdout = self.runCheckCommand(['--target', Test_Target, 'ls', '-a'], test_dir)
        self.assertIn('testing-dummy', stdout)

        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_uninstall(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install'], test_dir)
        self.assertTrue(os.path.exists(os.path.join(test_dir, 'yotta_modules', 'testing-dummy')))

        self.runCheckCommand(['uninstall', 'testing-dummy'], test_dir)
        self.assertFalse(os.path.exists(os.path.join(test_dir, 'yotta_modules', 'testing-dummy')))

        stdout = self.runCheckCommand(['--target', Test_Target, 'ls', '-a'], test_dir)
        self.assertNotIn(' testing-dummy', stdout)

        rmRf(test_dir)

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_uninstallNonExistent(self):
        test_dir = tempfile.mkdtemp()
        with open(os.path.join(test_dir, 'module.json'), 'w') as f:
            f.write(Test_Module_JSON)
        stdout = self.runCheckCommand(['--target', Test_Target, 'install'], test_dir)
        self.assertTrue(os.path.exists(os.path.join(test_dir, 'yotta_modules', 'testing-dummy')))

        stdout, stderr, statuscode = cli.run(['uninstall', 'nonexistent'], cwd=test_dir)
        self.assertNotEqual(statuscode, 0)

        rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "cannot build natively")
    def test_postInstall_topLevel(self):
        test_dir = util.writeTestFiles({})
        self.runCheckCommand(['--target', util.nativeTarget(), 'install', 'test-post-install'], test_dir)
        output = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], os.path.join(test_dir, 'test-post-install'))
        self.assertIn('post-install generated file compiled', output)
        self.assertIn('post-install generated header file included', output)
        self.assertIn('generated .cmake file included', output)

    @unittest.skipIf(not util.canBuildNatively(), "cannot build natively")
    def test_postInstall_dependency(self):
        test_dir = util.writeTestFiles({
'module.json':'''{
  "name": "test-postinstall",
  "version": "1.0.0",
  "license": "Apache-2.0",
  "dependencies": {
    "test-post-install": "*"
  },
  "bin":"./source"
}''',

'source/lib.c':'''
#include <stdio.h>
#include "test-post-install/generated.h"
int main(){ printf("generated return val=%d\\n", postInstallGenerated()); return 0; }
'''})
        output = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn('post-install generated file compiled', output)
        self.assertIn('post-install generated header file included', output)
        self.assertIn('generated .cmake file included', output)

    def runCheckCommand(self, args, test_dir):
        stdout, stderr, statuscode = cli.run(args, cwd=test_dir)
        #print stdout
        self.assertEqual(statuscode, 0)
        return stdout + stderr


if __name__ == '__main__':
    unittest.main()






# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import os
import logging
import re

# validate, , validate things, internal
from yotta.lib import validate
# Target, , represents an installed target, internal
from yotta.lib import target
# fsutils, , misc filesystem utils, internal
from yotta.lib import fsutils
# build, , build subcommand, internal
from yotta import build
# --config option, , , internal
from yotta import options


def addOptions(parser):
    options.config.addTo(parser)
    parser.add_argument(
        "--list", '-l', dest='list_only', default=False, action='store_true',
        help='List the tests that would be run, but don\'t run them. Implies --no-build'
    )
    parser.add_argument(
        "--no-build", '-n', dest='build', default=True, action='store_false',
        help='Don\'t build first.'
    )
    parser.add_argument(
        "tests", metavar='TEST_TO_RUN', nargs='*', type=str, default=[],
        help='List tests to run (omit to run the default set, or use "all" to run all).'
    )

    # relevant build options:
    parser.add_argument('-r', '--release-build', dest='release_build', action='store_true', default=True)
    parser.add_argument('-d', '--debug-build', dest='release_build', action='store_false', default=True)
    target.DerivedTarget.addBuildOptions(parser)


def findCTests(builddir, recurse_yotta_modules=False):
    ''' returns a list of (directory_path, [list of tuples of (test name, test command)]) '''
    # we don't run ctest -N to get the list of tests because unfortunately it
    # only lists the names, not the test commands. The best way to get at these
    # seems to be to parse the CTestTestfile.cmake files, which kinda sucks,
    # but works... Patches welcome.
    tests = []
    add_test_re = re.compile('add_test\\(([^" ]*)\s*"(.*)"\\)', flags=re.IGNORECASE)
    for root, dirs, files in os.walk(builddir, topdown=True):
        if not recurse_yotta_modules:
            dirs = [d for d in dirs if d != 'ym']
        if 'CTestTestfile.cmake' in files:
            with open(os.path.join(root, 'CTestTestfile.cmake'), 'r') as ctestf:
                dir_tests = []
                for line in ctestf:
                    if line.lower().startswith('add_test'):
                        match = add_test_re.search(line)
                        if match:
                            dir_tests.append((match.group(1), match.group(2)))
                        else:
                            logging.error(
                                "unknown CTest Syntax '%s', please report this error at http://github.com/ARMmbed/yotta/issues" %
                                line.rstrip('\n')
                            )
                if len(dir_tests):
                    tests.append((root, dir_tests))
    return tests

def moduleFromDirname(build_subdir, all_modules, toplevel_module):
    modtop = True
    submod = False
    module = toplevel_module
    # <topdir> /ym/<submod>/ym/<submod2>/somedir/somedir --> submod2
    for part in fsutils.fullySplitPath(build_subdir):
        if submod:
            if part in all_modules:
                module = all_modules[part]
            modtop = True
            submod = False
        else:
            if part == 'ym' and modtop:
                submod = True
            else:
                submod = False
                modtop = False
    return module

def execCommand(args, following_args):
    # remove the pseudo-name 'all': it wouldn't be recognised by build/cmake
    all_tests = 'all' in args.tests
    if all_tests:
        args.tests.remove('all')

    returncode = 0
    if args.build and not args.list_only:
        # we need to build before testing, make sure that any tests needed are
        # built:
        if all_tests:
            vars(args)['build_targets'] = args.tests + ['all_tests']
        else:
            vars(args)['build_targets'] = args.tests
        build_status = build.installAndBuild(args, following_args)
        # a generate or build step failure is fatal, but an install-step
        # failure should not prevent attempting tests:
        if build_status.get('generate_status', 0) != 0 or \
           build_status.get('build_status', 0) != 0 or \
           build_status.get('missing_status', 0) != 0:
            return 1
        else:
            returncode = build_status['status']

    cwd = os.getcwd()

    c = validate.currentDirectoryModule()
    if not c:
        return 1

    target, errors = c.satisfyTarget(args.target, additional_config=args.config)
    if errors:
        for error in errors:
            logging.error(error)
        return 1

    all_modules = c.getDependenciesRecursive(
                      target = target,
        available_components = [(c.getName(), c)],
                        test = True
    )


    builddir = os.path.join(cwd, 'build', target.getName())

    # get the list of tests we need to run, if --all is specified we also run
    # the tests for all of our submodules, otherwise we just run the tests for
    # this module.
    # If we have specific test specified, we also need to search for all the
    # tests, in case the specific test does not belong to this module
    tests = findCTests(builddir, recurse_yotta_modules=(all_tests or len(args.tests)))

    errcode = c.runScript('preTest')
    if errcode:
        return errcode

    passed = 0
    failed = 0
    for dirname, test_definitions in tests:
        module = moduleFromDirname(os.path.relpath(dirname, builddir), all_modules, c)
        logging.debug('inferred module %s from path %s', module.getName(), os.path.relpath(dirname, builddir))
        if (not len(args.tests)) and (module is not c) and not all_tests:
            continue
        info_filter = True
        filter_command = module.getScript('testReporter')
        for test_name, test_command in test_definitions:
            if len(args.tests) and not test_name in args.tests:
                logging.debug('skipping not-listed test %s: %s', test_name, test_command)
                continue
            if info_filter and filter_command:
                info_filter = False
                logging.info('using filter "%s" for tests in %s', ' '.join(filter_command), dirname)
            logging.info('test %s: %s', module.getName(), test_name)
            if args.list_only:
                continue
            test_returncode = target.test(
                       test_dir = dirname,
                     module_dir = module.path,
                   test_command = test_command,
                 filter_command = filter_command,
                   forward_args = following_args
            )
            if test_returncode:
                logging.error('test %s failed (command: %s)', test_name, test_command)
                failed += 1
                if not returncode:
                    returncode = 1
            else:
                logging.info('test %s passed', test_name)
                passed += 1
    if not args.list_only:
        logging.info("tests complete: %d passed, %d failed", passed, failed)

    return returncode




# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import logging
import os

# fsutils, , misc filesystem utils, internal
from yotta.lib import fsutils
# validate, , validate things, internal
from yotta.lib import validate


def addOptions(parser):
    parser.add_argument('component',
        help='Name of the dependency to remove'
    )

def execCommand(args, following_args):
    err = validate.componentNameValidationError(args.component)
    if err:
        logging.error(err)
        return 1
    c = validate.currentDirectoryModule()
    if not c:
        return 1
    status = 0
    if not c.removeDependency(args.component):
        status = 1
    else:
        c.writeDescription()
    path = os.path.join(c.modulesPath(), args.component)
    if fsutils.isLink(path):
        fsutils.rmF(path)
    else:
        fsutils.rmRf(path)
    return status




# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import os
import logging

# validate, , validate things, internal
from yotta.lib import validate
# --config option, , , internal
from yotta import options


def addOptions(parser):
    options.config.addTo(parser)
    parser.add_argument('program', default=None, nargs='?',
        help='name of the program to be debugged'
    )


def execCommand(args, following_args):
    cwd = os.getcwd()

    c = validate.currentDirectoryModule()
    if not c:
        return 1

    target, errors = c.satisfyTarget(args.target, additional_config=args.config)
    if errors:
        for error in errors:
            logging.error(error)
        return 1

    builddir = os.path.join(cwd, 'build', target.getName())

    if args.program is None:
        if c.isApplication():
            # if no program was specified, default to the name of the executable
            # module (if this is an executable module)
            args.program = c.getName()
        else:
            logging.error('This module describes a library not an executable, so you must name an executable to debug.')
            return 1

    errcode = c.runScript('preDebug', {"YOTTA_PROGRAM":args.program})
    if errcode:
        return errcode

    error = target.debug(builddir, args.program)
    if error:
        logging.error(error)
        errcode = 1

    return errcode





#!/usr/bin/env python
# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest
import os
import copy

# internal modules:
from yotta.test.cli import cli
from yotta.test.cli import util

Test_Target = "x86-osx-native,*"
Test_Target_Name = 'x86-osx-native'
Test_Target_Old_Version = '0.0.7'

Test_Shrinkwrap = {
'module.json':'''{
  "name": "test-shrinkwrap",
  "version": "0.0.0",
  "description": "Test yotta shrinkwrap",
  "author": "James Crosby <james.crosby@arm.com>",
  "license": "Apache-2.0",
  "dependencies":{
    "test-testing-dummy": "*"
  }
}''',
'source/foo.c':'''#include "stdio.h"
int foo(){
    printf("foo!\\n");
    return 7;
}''',
# test-testing-dummy v0.0.1 (a newer version is available from the registry)
'yotta_modules/test-testing-dummy/module.json':'''{
  "name": "test-testing-dummy",
  "version": "0.0.1",
  "description": "Test yotta's compilation of tests.",
  "author": "James Crosby <james.crosby@arm.com>",
  "license": "Apache-2.0"
}
'''
}

Test_Shrinkwrap_Missing_Dependency = {
'module.json':'''{
  "name": "test-shrinkwrap",
  "version": "0.0.0",
  "description": "Test yotta shrinkwrap",
  "author": "James Crosby <james.crosby@arm.com>",
  "license": "Apache-2.0",
  "dependencies":{
    "test-testing-dummy": "*"
  }
}''',
'source/foo.c':'''#include "stdio.h"
int foo(){
    printf("foo!\\n");
    return 7;
}'''
}

Test_Existing_Shrinkwrap_Missing_Dependency = copy.copy(Test_Shrinkwrap_Missing_Dependency)
Test_Existing_Shrinkwrap_Missing_Dependency['yotta-shrinkwrap.json'] = '''
{
  "modules": [
    {
      "version": "0.0.0",
      "name": "test-shrinkwrap"
    },
    {
      "version": "0.0.1",
      "name": "test-testing-dummy"
    }
  ],
  "targets": [
    {
      "version": "%s",
      "name": "%s"
    }
  ]
}''' % (Test_Target_Old_Version, Test_Target_Name)

Test_Existing_Shrinkwrap = copy.copy(Test_Shrinkwrap)
Test_Existing_Shrinkwrap['yotta_targets/inherits-from-test-target/target.json'] = '''{
  "name": "inherits-from-test-target",
  "version": "1.0.0",
  "license": "Apache-2.0",
  "inherits": {
     "%s": "*"
  }
}''' % Test_Target_Name
Test_Existing_Shrinkwrap['yotta-shrinkwrap.json'] = '''
{
  "modules": [
    {
      "version": "0.0.0",
      "name": "test-shrinkwrap"
    },
    {
      "version": "0.0.1",
      "name": "test-testing-dummy"
    }
  ],
  "targets": [
    {
      "version": "%s",
      "name": "%s"
    }
  ]
}''' % (Test_Target_Old_Version, Test_Target_Name)

class TestCLIShrinkwrap(unittest.TestCase):

    def testCreateShrinkwrap(self):
        test_dir = util.writeTestFiles(Test_Shrinkwrap, True)

        stdout, stderr, statuscode = cli.run(['-t', Test_Target, '--plain', 'shrinkwrap'], cwd=test_dir)
        self.assertEqual(statuscode, 0)
        self.assertTrue(os.path.exists(os.path.join(test_dir, 'yotta-shrinkwrap.json')))

        util.rmRf(test_dir)

    def testMissingDependenciesShrinkwrap(self):
        test_dir = util.writeTestFiles(Test_Shrinkwrap_Missing_Dependency, True)
        stdout, stderr, statuscode = cli.run(['-t', Test_Target, '--plain', 'shrinkwrap'], cwd=test_dir)
        self.assertNotEqual(statuscode, 0)
        self.assertFalse(os.path.exists(os.path.join(test_dir, 'yotta-shrinkwrap.json')))
        self.assertIn('is missing', stdout+stderr)
        util.rmRf(test_dir)

    def testInstallWithShrinkwrap(self):
        test_dir = util.writeTestFiles(Test_Existing_Shrinkwrap_Missing_Dependency, True)
        stdout, stderr, statuscode = cli.run(['-t', Test_Target, '--plain', 'install'], cwd=test_dir)
        self.assertEqual(statuscode, 0)

        stdout, stderr, statuscode = cli.run(['-t', Test_Target, '--plain', 'list'], cwd=test_dir)
        self.assertEqual(statuscode, 0)
        # as opposed to 0.0.2 which is the latest
        self.assertIn('test-testing-dummy 0.0.1', stdout+stderr)

        stdout, stderr, statuscode = cli.run(['-t', Test_Target, '--plain', 'target'], cwd=test_dir)
        self.assertEqual(statuscode, 0)
        self.assertIn('%s %s' % (Test_Target_Name, Test_Target_Old_Version), stdout+stderr)

        util.rmRf(test_dir)

    def testBaseTargetInstallWithShrinkwrap(self):
        test_dir = util.writeTestFiles(Test_Existing_Shrinkwrap, True)
        stdout, stderr, statuscode = cli.run(['-t', 'inherits-from-test-target', '--plain', 'install'], cwd=test_dir)
        self.assertEqual(statuscode, 0)

        stdout, stderr, statuscode = cli.run(['-t', 'inherits-from-test-target', '--plain', 'target'], cwd=test_dir)
        self.assertEqual(statuscode, 0)
        self.assertIn('%s %s' % (Test_Target_Name, Test_Target_Old_Version), stdout+stderr)

        util.rmRf(test_dir)

    def testUpdateWithShrinkwrap(self):
        test_dir = util.writeTestFiles(Test_Existing_Shrinkwrap, True)
        stdout, stderr, statuscode = cli.run(['-t', Test_Target, '--plain', 'update'], cwd=test_dir)
        self.assertEqual(statuscode, 0)

        stdout, stderr, statuscode = cli.run(['-t', Test_Target, '--plain', 'list'], cwd=test_dir)
        self.assertEqual(statuscode, 0)
        # as opposed to 0.0.2 which is the latest
        self.assertIn('test-testing-dummy 0.0.1', stdout+stderr)

        util.rmRf(test_dir)




#!/usr/bin/env python
# Copyright 2016 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import unittest
import os

# internal modules:
from yotta.test.cli import util
from yotta.test.cli import cli

def _nopDebugTargetDescription(name):
    native_target = util.nativeTarget()
    if ',' in native_target:
        native_target = native_target[:native_target.find(',')]
    return {
    'target.json':'''{
      "name": "%s",
      "version": "1.0.0",
      "license": "Apache-2.0",
      "inherits": {
        "%s": "*"
      },
      "scripts": {
        "debug": ["./scripts/nop.py", "$program"]
      }
    }
    ''' % (name, native_target),
    './scripts/nop.py':'''
import os
print('would debug %s' % os.environ['YOTTA_PROGRAM'])
    '''
    }

class TestCLIDebug(unittest.TestCase):
    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_noop_debug(self):
        test_dir = util.writeTestFiles(util.Test_Trivial_Exe, True)
        util.writeTestFiles(_nopDebugTargetDescription('debug-test-target'), test_dir=os.path.join(test_dir, 'yotta_targets', 'debug-test-target'))
        output = util.runCheckCommand(['--target', 'debug-test-target', 'build'], test_dir)
        output = util.runCheckCommand(['--target', 'debug-test-target', 'debug'], test_dir)
        self.assertIn('would debug source/test-trivial-exe', output)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_notfound_debug(self):
        test_dir = util.writeTestFiles(util.Test_Trivial_Exe, True)
        target_descr = _nopDebugTargetDescription('debug-test-target')
        del target_descr['./scripts/nop.py']
        util.writeTestFiles(target_descr, test_dir=os.path.join(test_dir, 'yotta_targets', 'debug-test-target'))

        # in this case, without the script present we expect a failure
        output = util.runCheckCommand(['--target', 'debug-test-target', 'build'], test_dir)
        stdout, stderr, statuscode = cli.run(['--target', 'debug-test-target', 'debug'], cwd=test_dir)
        self.assertNotEqual(statuscode, 0)
        util.rmRf(test_dir)





# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import os
import signal
import subprocess
import logging
import string
import traceback
import errno
import itertools
import shlex
from collections import OrderedDict

# Ordered JSON, , read & write json, internal
from yotta.lib import ordered_json
# Pack, , common parts of Components/Targets, internal
from yotta.lib import pack
from yotta.lib.pack import tryTerminate as _tryTerminate
# fsutils, , misc filesystem utils, internal
from yotta.lib import fsutils

Target_Description_File = 'target.json'
App_Config_File = 'config.json'
Registry_Namespace = 'targets'
Schema_File = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'schema', 'target.json')

logger = logging.getLogger('target')

def _ignoreSignal(signum, frame):
    logger.debug('ignoring signal %s, traceback:\n%s' % (
        signum, ''.join(traceback.format_list(traceback.extract_stack(frame)))
    ))

def _newPGroup():
    os.setpgrp()

def _mergeDictionaries(*args):
    ''' merge dictionaries of dictionaries recursively, with elements from
        dictionaries earlier in the argument sequence taking precedence
    '''
    # to support merging of OrderedDicts, copy the result type from the first
    # argument:
    result = type(args[0])()
    for k, v in itertools.chain(*[x.items() for x in args]):
        if not k in result:
            result[k] = v
        elif isinstance(result[k], dict) and isinstance(v, dict):
            result[k] = _mergeDictionaries(result[k], v)
    return result

def _mirrorStructure(dictionary, value):
    ''' create a new nested dictionary object with the same structure as
        'dictionary', but with all scalar values replaced with 'value'
    '''
    result = type(dictionary)()
    for k in dictionary.keys():
        if isinstance(dictionary[k], dict):
            result[k] = _mirrorStructure(dictionary[k], value)
        else:
            result[k] = value
    return result

# API

def loadAdditionalConfig(config_path):
    ''' returns (error, config)
    '''
    error = None
    config = {}
    if not config_path:
        return (error, config)
    if os.path.isfile(config_path):
        try:
            config = ordered_json.load(config_path)
        except Exception as e:
            error = "Invalid syntax in file %s: %s" % (config_path, e)
    else:
        # try to interpret the argument as literal JSON
        try:
            config = ordered_json.loads(config_path)
        except Exception as e:
            # if this fails too, guess whether it was intended to be JSON or
            # not, and display an appropriate error message
            if '{' in config_path or '}' in config_path:
                error = "Invalid syntax in literal JSON: %s" % e
            else:
                error = "File \"%s\" does not exist" % config_path
    logger.debug('read additional config: %s', config)
    return (error, config)

def getDerivedTarget(
        target_name_and_version,
                   targets_path,
                application_dir = None,
                install_missing = True,
               update_installed = False,
              additional_config = None,
                     shrinkwrap = None
    ):
    # access, , get components, internal
    from yotta.lib import access
    from yotta.lib import access_common
    ''' Get the specified target description, optionally ensuring that it (and
        all dependencies) are installed in targets_path.

        Returns (DerivedTarget, errors), or (None, errors) if the leaf target
        could not be found/installed.
    '''
    logger.debug('satisfy target: %s' % target_name_and_version);
    if ',' in target_name_and_version:
        name, version_req = target_name_and_version.split(',')
    else:
        name = target_name_and_version
        version_req = '*'

    # shrinkwrap is the raw json form, not mapping form here, so rearrange it
    # before indexing:
    if shrinkwrap is not None:
        shrinkwrap_version_req = {
            x['name']: x['version'] for x in shrinkwrap.get('targets', [])
        }.get(name, None)
    else:
        shrinkwrap_version_req = None
    if shrinkwrap_version_req is not None:
        logger.debug(
            'respecting shrinkwrap version %s for %s', shrinkwrap_version_req, name
        )

    dspec = pack.DependencySpec(
                                 name,
                                 version_req,
        shrinkwrap_version_req = shrinkwrap_version_req
    )

    leaf_target      = None
    previous_name    = dspec.name
    search_dirs      = [targets_path]
    target_hierarchy = []
    errors           = []
    while True:
        t = None
        try:
            if install_missing:
                t = access.satisfyVersion(
                                 name = dspec.name,
                     version_required = dspec.versionReq(),
                            available = target_hierarchy,
                         search_paths = search_dirs,
                    working_directory = targets_path,
                     update_installed = ('Update' if update_installed else None),
                                 type = 'target',
                   inherit_shrinkwrap = shrinkwrap
                )
            else:
                t = access.satisfyVersionFromSearchPaths(
                                 name = dspec.name,
                     version_required = dspec.versionReq(),
                         search_paths = search_dirs,
                                 type = 'target',
                   inherit_shrinkwrap = shrinkwrap
                )
        except access_common.Unavailable as e:
            errors.append(e)
        if not t:
            if install_missing:
                logger.error(
                    'could not install target %s for %s' %
                    (dspec, previous_name)
                )
            break
        else:
            target_hierarchy.append(t)
            previous_name = dspec.name
            assert(isinstance(t, Target))
            dspec = t.baseTargetSpec() #pylint: disable=no-member
            if not leaf_target:
                leaf_target = t
            if dspec is None:
                break
    if leaf_target is None:
        return (None, errors)
    # if we have a valid target, try to load the app-specific config data (if
    # any):
    app_config = {}
    if application_dir is not None:
        app_config_fname = os.path.join(application_dir, App_Config_File)
        if os.path.exists(app_config_fname):
            try:
                app_config = ordered_json.load(app_config_fname)
            except Exception as e:
                errors.append(Exception("Invalid application config.json: %s" % (e)))
    return (DerivedTarget(leaf_target, target_hierarchy[1:], app_config, additional_config), errors)

class Target(pack.Pack):
    def __init__(
            self,
            path,
            installed_linked = False,
            latest_suitable_version = None,
            inherit_shrinkwrap = None
        ):
        ''' Initialise a Target based on a directory. If the directory does not
            contain a valid target.json file the initialised object will test
            false, and will contain an error property containing the failure.
        '''
        # re-initialise with the information from the most-derived target
        super(Target, self).__init__(
                                      path,
               description_filename = Target_Description_File,
                   installed_linked = installed_linked,
                    schema_filename = Schema_File,
            latest_suitable_version = latest_suitable_version,
                 inherit_shrinkwrap = inherit_shrinkwrap
        )
        if self.description and inherit_shrinkwrap is not None:
            # when inheriting a shrinkwrap, check that this module is
            # listed in the shrinkwrap, otherwise emit a warning:
            if next((x for x in inherit_shrinkwrap.get('targets', []) if x['name'] == self.getName()), None) is None:
                logger.warning("%s missing from shrinkwrap", self.getName())

    def baseTargetSpec(self):
        ''' returns pack.DependencySpec for the base target of this target (or
            None if this target does not inherit from another target.
        '''
        inherits = self.description.get('inherits', {})
        if len(inherits) == 1:
            name, version_req = list(inherits.items())[0]
            shrinkwrap_version_req = self.getShrinkwrapMapping('targets').get(name, None)
            if shrinkwrap_version_req is not None:
                logger.debug(
                    'respecting shrinkwrap version %s for %s', shrinkwrap_version_req, name
                )
            return pack.DependencySpec(
                name,
                version_req,
                shrinkwrap_version_req = shrinkwrap_version_req
            )
        elif len(inherits) > 1:
            logger.error('target %s specifies multiple base targets, but only one is allowed', self.getName())
        return None

    def getRegistryNamespace(self):
        return Registry_Namespace

    def getConfig(self):
        return self.description.get('config', OrderedDict())

class DerivedTarget(Target):
    def __init__(self, leaf_target, base_targets, app_config, additional_config):
        ''' Initialise a DerivedTarget (representing an inheritance hierarchy of
            Targets.), given the most-derived Target description, and a set of
            available Targets to compose the rest of the lineage from.

            DerivedTarget provides build & debug commands, and access to the
            derived target config info (merged with the application config
            info from config.json, if any).

            It's possible to update the application config for an existing
            DerivedTarget instance.

            DerivedTarget can also be used as a stand-in for the most-derived
            (leaf) target in the inheritance hierarchy.
        '''

        # initialise the base class as a copy of leaf_target
        super(DerivedTarget, self).__init__(
                               path = leaf_target.path,
                   installed_linked = leaf_target.installed_linked,
            latest_suitable_version = leaf_target.latest_suitable_version
        )

        self.hierarchy = [leaf_target] + base_targets[:]
        self.config = None
        self.config_blame = None
        self.app_config = app_config
        self.additional_config = additional_config or {}


    # override truthiness to test validity of the entire hierarchy:
    def __nonzero__(self):
        for t in self.hierarchy:
            if not t: return False
        return bool(len(self.hierarchy))
    def __bool__(self):
        return self.__nonzero__()

    def getScript(self, scriptname):
        ''' return the specified script if one exists (possibly inherited from
            a base target)
        '''
        for t in self.hierarchy:
            s = t.getScript(scriptname)
            if s:
                return s
        return None

    def _loadConfig(self):
        ''' load the configuration information from the target hierarchy '''
        config_dicts = [self.additional_config, self.app_config] + [t.getConfig() for t in self.hierarchy]
        # create an identical set of dictionaries, but with the names of the
        # sources in place of the values. When these are merged they will show
        # where each merged property came from:
        config_blame = [
            _mirrorStructure(self.additional_config, 'command-line config'),
            _mirrorStructure(self.app_config, 'application\'s config.json'),
        ] + [
            _mirrorStructure(t.getConfig(), t.getName()) for t in self.hierarchy
        ]

        self.config = _mergeDictionaries(*config_dicts)
        self.config_blame = _mergeDictionaries(*config_blame)
        # note that backwards compatibility with the "similarTo" data that used
        # to be used for target-dependencies is ensured at the point of use. We
        # don't merge similarTo into the config because it might break things
        # in the config (clobber objects with scalar values, for example)

    def _ensureConfig(self):
        if self.config is None:
            self._loadConfig()

    def getConfigValue(self, conf_key):
        self._ensureConfig()
        # jsonpointer, pip install jsonpointer, BSD 3 Clause
        import jsonpointer
        try:
            return jsonpointer.resolve_pointer(self.config, conf_key)
        except jsonpointer.JsonPointerException as e:
            # fall back to legacy dot-separated pointers
            key_path = conf_key.split('.');
            c = self.config
            for part in key_path:
                if part in c:
                    c = c[part]
                else:
                    return None
            return c

    def getSimilarTo_Deprecated(self):
        r = []
        for t in self.hierarchy:
            r.append(t.getName())
            r += t.description.get('similarTo', [])
        return r

    def getMergedConfig(self):
        self._ensureConfig()
        return self.config

    def getConfigBlame(self):
        self._ensureConfig()
        return self.config_blame

    def getToolchainFiles(self):
        ''' return a list of toolchain file paths in override order (starting
            at the bottom/leaf of the hierarchy and ending at the base).
            The list is returned in the order they should be included
            (most-derived last).
        '''
        return reversed([
            os.path.join(x.path, x.description['toolchain']) for x in self.hierarchy if 'toolchain' in x.description
        ])

    def getAdditionalIncludes(self):
        ''' Return the list of cmake files which are to be included by yotta in
            every module built. The list is returned in the order they should
            be included (most-derived last).
        '''
        return reversed([
            os.path.join(t.path, include_file)
                for t in self.hierarchy
                for include_file in t.description.get('cmakeIncludes', [])
        ])

    def inheritsFrom(self, target_name):
        ''' Return true if this target inherits from the named target (directly
            or indirectly. Also returns true if this target is the named
            target. Otherwise return false.
        '''
        for t in self.hierarchy:
            if t and t.getName() == target_name or target_name in t.description.get('inherits', {}):
                return True
        return False

    @classmethod
    def addBuildOptions(cls, parser):
        parser.add_argument('-G', '--cmake-generator', dest='cmake_generator',
           default='Ninja',
           help='CMake generator to use (defaults to Ninja). You can use this '+
           'to generate IDE project files instead, see cmake --help for '+
           'possible generator names. Note that only Ninja or Unix Makefile '+
           'based generators will work correctly with yotta.',
           metavar='CMAKE_GENERATOR',
           type=str
        )

    @classmethod
    def _findNinja(cls):
        # sometimes ninja is called ninja-build
        for name in ('ninja', 'ninja-build'):
            if fsutils.which(name) is not None:
                return name
        # default to ninja:
        return 'ninja'

    @classmethod
    def overrideBuildCommand(cls, generator_name, targets=None):
        if targets is None:
            targets = []
        # when we build using cmake --build, the nice colourised output is lost
        # - so override with the actual build command for command-line
        # generators where people will care:
        try:
            r = {
                'Unix Makefiles': ['make'],
                'Ninja': [cls._findNinja()]
            }[generator_name]
            # all of the above build programs take the build targets (e.g.
            # "all") as the last arguments
            if targets is not None:
                r += targets
            return r
        except KeyError:
            return None

    def hintForCMakeGenerator(self, generator_name, component):
        if generator_name in ('Ninja', 'Unix Makefiles'):
            return None
        try:
            name = self.getName()
            component_name = component.getName()
            return {
                'Xcode':
                    'a project file has been generated at ./build/%s/%s.xcodeproj' % (name, component_name),
                'Sublime Text 2 - Ninja':
                    'a project file has been generated at ./build/%s/%s.sublime-project' % (name, component_name),
                'Sublime Text 2 - Unix Makefiles':
                    'a project file has been generated at ./build/%s/%s.sublime-project' % (name, component_name),
                'Eclipse CDT4 - Ninja':
                    'a project file has been generated at ./build/%s/.project' % name,
                'Eclipse CDT4 - Unix Makefiles':
                    'a project file has been generated at ./build/%s/.project' % name
            }[generator_name]
        except KeyError:
            return 'project files for %s have been generated in ./build/%s' % (component_name, name)

    def exec_helper(self, cmd, builddir):
        ''' Execute the given command, returning an error message if an error occured
            or None if the command was succesful.'''
        try:
            child = subprocess.Popen(cmd, cwd=builddir)
            child.wait()
        except OSError as e:
            if e.errno == errno.ENOENT:
                if cmd[0] == 'cmake':
                    return 'CMake is not installed, please follow the installation instructions at http://yottadocs.mbed.com/#installing'
                else:
                    return '%s is not installed' % (cmd[0])
            else:
                return 'command %s failed' % (cmd)
        if child.returncode:
            return 'command %s failed' % (cmd)

    @fsutils.dropRootPrivs
    def build(self, builddir, component, args, release_build=False, build_args=None, targets=None):
        ''' Execute the commands necessary to build this component, and all of
            its dependencies. '''
        if build_args is None:
            build_args = []
        if targets is None:
            targets = []
        # in the future this may be specified in the target description, but
        # for now we only support cmake, so everything is simple:
        build_type = ('Debug', 'RelWithDebInfo')[release_build]
        if build_type:
            cmd = ['cmake', '-D', 'CMAKE_BUILD_TYPE=%s' % build_type, '-G', args.cmake_generator, '.']
        else:
            cmd = ['cmake', '-G', args.cmake_generator, '.']
        res = self.exec_helper(cmd, builddir)
        if res is not None:
            return res

        # work-around various yotta-specific issues with the generated
        # Ninja/project files:
        from yotta.lib import cmake_fixups
        cmake_fixups.applyFixupsForFenerator(args.cmake_generator, builddir, component)

        build_command = self.overrideBuildCommand(args.cmake_generator, targets=targets)
        if build_command:
            cmd = build_command + build_args
        else:
            cmd = ['cmake', '--build', builddir]
            if len(targets):
                # !!! FIXME: support multiple targets with the default CMake
                # build command
                cmd += ['--target', targets[0]]
            cmd += build_args
        res = self.exec_helper(cmd, builddir)
        if res is not None:
            return res
        hint = self.hintForCMakeGenerator(args.cmake_generator, component)
        if hint:
            logger.info(hint)

    def findProgram(self, builddir, program):
        ''' Return the builddir-relative path of program, if only a partial
            path is specified. Returns None and logs an error message if the
            program is ambiguous or not found
	'''
        # if this is an exact match, do no further checking:
        if os.path.isfile(os.path.join(builddir, program)):
            logging.info('found %s' % program)
            return program
        exact_matches = []
        insensitive_matches = []
        approx_matches = []
        for path, dirs, files in os.walk(builddir):
            if program in files:
                exact_matches.append(os.path.relpath(os.path.join(path, program), builddir))
                continue
            files_lower = [f.lower() for f in files]
            if program.lower() in files_lower:
                insensitive_matches.append(
                    os.path.relpath(
                        os.path.join(path, files[files_lower.index(program.lower())]),
                        builddir
                    )
                )
                continue
            # !!! TODO: in the future add approximate string matching (typos,
            # etc.), for now we just test stripping any paths off program, and
            # looking for substring matches:
            pg_basen_lower_noext = os.path.splitext(os.path.basename(program).lower())[0]
            for f in files_lower:
                if pg_basen_lower_noext in f:
                    approx_matches.append(
                        os.path.relpath(
                            os.path.join(path, files[files_lower.index(f)]),
                            builddir
                        )
                    )

        if len(exact_matches) == 1:
            logging.info('found %s at %s', program, exact_matches[0])
            return exact_matches[0]
        elif len(exact_matches) > 1:
            logging.error(
                '%s matches multiple executables, please use a full path (one of %s)' % (
                    program,
                    ', or '.join(['"'+os.path.join(m, program)+'"' for m in exact_matches])
                )
            )
            return None
        # if we have matches with and without a file extension, prefer the
        # no-file extension version, and discard the others (so we avoid
        # picking up post-processed files):
        reduced_approx_matches = []
        for m in approx_matches:
            root = os.path.splitext(m)[0]
            if (m == root) or (root not in approx_matches):
                reduced_approx_matches.append(m)
        approx_matches = reduced_approx_matches

        for matches in (insensitive_matches, approx_matches):
            if len(matches) == 1:
                logging.info('found %s at %s' % (
                    program, matches[0]
                ))
                return matches[0]
            elif len(matches) > 1:
                logging.error(
                    '%s is similar to several executables found. Please use an exact name:\n%s' % (
                        program,
                        '\n'.join(matches)
                    )
                )
                return None
        logging.error('could not find program "%s" to debug' %  program)
        return None

    @fsutils.dropRootPrivs
    def start(self, builddir, program, forward_args):
        ''' Launch the specified program. Uses the `start` script if specified
            by the target, attempts to run it natively if that script is not
            defined.
        '''
        child = None
        try:
            prog_path = self.findProgram(builddir, program)
            if prog_path is None:
                return

            env = os.environ.copy()
            env['YOTTA_PROGRAM'] = prog_path

            if self.getScript('start'):
                cmd = [
                    os.path.expandvars(string.Template(x).safe_substitute(program=prog_path))
                    for x in self.getScript('start')
                ] + forward_args
            else:
                cmd = shlex.split('./' + prog_path) + forward_args

            logger.debug('starting program: %s', cmd)
            child = subprocess.Popen(
                cmd, cwd = builddir, env = env
            )
            child.wait()
            if child.returncode:
                return "process exited with status %s" % child.returncode
            child = None
        except OSError as e:
            import errno
            if e.errno == errno.ENOEXEC:
                return ("the program %s cannot be run (perhaps your target "+
                        "needs to define a 'start' script to start it on its "
                        "intended execution target?)") % prog_path
        finally:
            if child is not None:
                _tryTerminate(child)

    def debug(self, builddir, program):
        ''' Launch a debugger for the specified program. Uses the `debug`
            script if specified by the target, falls back to the `debug` and
            `debugServer` commands if not. `program` is inserted into the
            $program variable in commands.
        '''
        try:
            signal.signal(signal.SIGINT, _ignoreSignal);
            if self.getScript('debug') is not None:
                return self._debugWithScript(builddir, program)
            elif 'debug' in self.description:
                logger.warning(
                    'target %s provides deprecated debug property. It should '+
                    'provide script.debug instead.', self.getName()

                )
                return self._debugDeprecated(builddir, program)
            else:
                return "Target %s does not specify debug commands" % self
        finally:
            # clear the sigint handler
            signal.signal(signal.SIGINT, signal.SIG_DFL);

    @fsutils.dropRootPrivs
    def _debugWithScript(self, builddir, program):
        child = None
        try:
            prog_path = self.findProgram(builddir, program)
            if prog_path is None:
                return

            env = os.environ.copy()
            env['YOTTA_PROGRAM'] = prog_path

            cmd = [
                os.path.expandvars(string.Template(x).safe_substitute(program=prog_path))
                for x in self.getScript('debug')
            ]
            logger.debug('starting debugger: %s', cmd)
            child = subprocess.Popen(
                cmd, cwd = builddir, env = env
            )
            child.wait()
            if child.returncode:
                return "debug process exited with status %s" % child.returncode
            child = None
        except:
            # reset the terminal, in case the debugger has screwed it up
            os.system('reset')
            raise
        finally:
            if child is not None:
                _tryTerminate(child)

    @fsutils.dropRootPrivs
    def _debugDeprecated(self, builddir, program):
        prog_path = self.findProgram(builddir, program)
        if prog_path is None:
            return

        with open(os.devnull, "w") as dev_null:
            daemon = None
            child = None
            try:
                # debug-server is the old name, debugServer is the new name
                debug_server_prop = 'debugServer'
                if not debug_server_prop in self.description:
                    debug_server_prop = 'debug-server'

                if debug_server_prop in self.description:
                    logger.debug('starting debug server...')
                    daemon = subprocess.Popen(
                                   self.description[debug_server_prop],
                             cwd = builddir,
                          stdout = dev_null,
                          stderr = dev_null,
                      preexec_fn = _newPGroup
                    )
                else:
                    daemon = None

                cmd = [
                    os.path.expandvars(string.Template(x).safe_substitute(program=prog_path))
                    for x in self.description['debug']
                ]
                logger.debug('starting debugger: %s', cmd)
                child = subprocess.Popen(
                    cmd, cwd = builddir
                )
                child.wait()
                if child.returncode:
                    return "debug process executed with status %s" % child.returncode
                child = None
            except:
                # reset the terminal, in case the debugger has screwed it up
                os.system('reset')
                raise
            finally:
                if child is not None:
                    try:
                        child.terminate()
                    except OSError as e:
                        pass
                if daemon is not None:
                    logger.debug('shutting down debug server...')
                    try:
                        daemon.terminate()
                    except OSError as e:
                        pass

    @fsutils.dropRootPrivs
    def test(self, test_dir, module_dir, test_command, filter_command, forward_args):
        # we assume that test commands are relative to the current directory
        # (filter commands are relative to the module dir to make it possible
        # to use filter scripts shipped with the module)
        test_command = './' + test_command
        test_script = self.getScript('test')
        if test_script is None:
            cmd = shlex.split(test_command) + forward_args
        else:
            cmd = [
                os.path.expandvars(string.Template(x).safe_substitute(program=os.path.abspath(os.path.join(test_dir, test_command))))
                for x in test_script
            ] + forward_args

        # if the command is a python script, run it with the python interpreter
        # being used to run yotta:
        if test_command[0].lower().endswith('.py'):
            import sys
            python_interpreter = sys.executable
            cmd = [python_interpreter] + cmd
        if filter_command and filter_command[0].lower().endswith('.py'):
            import sys
            python_interpreter = sys.executable
            filter_command = [python_interpreter] + filter_command

        env = os.environ.copy()
        env['YOTTA_PROGRAM'] = test_command

        test_child = None
        test_filter = None
        try:
            logger.debug('running test: %s', cmd)
            if filter_command:
                logger.debug('using output filter command: %s', filter_command)
                test_child = subprocess.Popen(
                    cmd, cwd = test_dir, stdout = subprocess.PIPE, env = env
                )
                try:
                    test_filter = subprocess.Popen(
                        filter_command, cwd = module_dir, stdin = test_child.stdout, env = env
                    )
                except OSError as e:
                    logger.error('error starting test output filter "%s": %s', filter_command, e)
                    _tryTerminate(test_child)
                    return 1
                logger.debug('waiting for filter process')
                test_filter.communicate()
                if test_child.poll() is None:
                    logger.warning('test child has not exited and will be terminated')
                    _tryTerminate(test_child)
                test_child.stdout.close()
                returncode = test_filter.returncode
                test_child = None
                test_filter = None
                if returncode:
                    logger.debug("test filter exited with status %s (=fail)", returncode)
                    return 1
            else:
                try:
                    test_child = subprocess.Popen(
                        cmd, cwd = test_dir, env = env
                    )
                    logger.debug('waiting for test child')
                except OSError as e:
                    if e.errno == errno.ENOENT:
                        logger.error('Error: no such file or directory: "%s"', cmd[0])
                        return 1
                    raise
                test_child.wait()
                returncode = test_child.returncode
                test_child = None
                if returncode:
                    logger.debug("test process exited with status %s (=fail)", returncode)
                    return 1
        finally:
            if test_child is not None:
                _tryTerminate(test_child)
            if test_filter is not None:
                _tryTerminate(test_filter)
        logger.debug("test %s passed", test_command)
        return 0



# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import argparse
import logging

# auth, , authenticate users, internal
from yotta.lib import auth
# Registry Access, , access modules in the registry, internal
from yotta.lib import registry_access
# --registry option, , , internal
from yotta import options

def addOptions(parser):
    # accept top-level registry option at subcommand level too
    options.registry.addTo(parser)
    options.noninteractive.addTo(parser)
    options.plain.addTo(parser)

    # pass the API key for logging into this registry (it will be saved, and
    # used for future requests to this registry)
    parser.add_argument('--apikey', '-k', dest='apikey', help=argparse.SUPPRESS)
    # --github can be used to force login with github instead of mbed, the
    # default): github login is necessary for pulling things from private
    # github repositories. You shouldn't normally need to do this, as you will
    # be prompted for github login automatically when required, but this is
    # useful if you want to pre-cache login information for any reason.
    parser.add_argument('--github', dest='github_login', action='store_true',
        default=False, help='force login with GitHub instead of mbed. You '+
        "shouldn't normally need to use this switch, as you will be prompted "+
        "for GitHub login automatically if required."
    )

def execCommand(args, following_args):
    registry = args.registry
    if args.apikey:
        registry_access.setAPIKey(registry, args.apikey)
    try:
        provider = None
        if args.github_login:
            provider = 'github'
        return auth.authorizeUser(registry, provider=provider, interactive=args.interactive)
    except auth.AuthTimedOut as e:
        logging.error("Login failed: %s", e)
        return 1




#!/usr/bin/env python
# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest
import os
import tempfile
import stat

# internal modules:
from yotta.lib.fsutils import rmRf
from yotta.test.cli import cli


Test_Module_JSON = '''{
  "name": "testmod",
  "version": "0.0.0",
  "description": "a test module",
  "keywords": [
  ],
  "author": "Someone Somewhere <someone.somewhere@yottabuild.org>",
  "repository": {
    "url": "git@github.com:mbedmicro/testmod.git",
    "type": "git"
  },
  "homepage": "https://github.com/mbedmicro/testmod",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "dependencies": {},
  "targetDependencies": {}
}
'''

Check_Not_Stat = stat.S_IRWXG | stat.S_IRWXO

class TestCLITarget(unittest.TestCase):
    def setUp(self):
        self.test_dir = tempfile.mkdtemp()
        with open(os.path.join(self.test_dir, 'module.json'), 'w') as f:
            f.write(Test_Module_JSON)

    def tearDown(self):
        rmRf(self.test_dir)

    def test_setTarget(self):
        rmRf(os.path.join(self.test_dir, '.yotta.json'))
        stdout = self.runCheckCommand(['target', 'testtarget', '-g', '-n'])
        stdout = self.runCheckCommand(['target'])
        self.assertTrue(stdout.find('testtarget') != -1)
        stdout = self.runCheckCommand(['target', 'x86-linux-native', '-g'])
        if os.name == 'posix':
            # check that the settings file was created with the right permissions
            self.assertFalse(
                os.stat(os.path.join(os.path.expanduser('~'), '.yotta', 'config.json')).st_mode & Check_Not_Stat
            )

    def test_setTargetLocal(self):
        stdout = self.runCheckCommand(['target', 'testtarget', '-n'])
        stdout = self.runCheckCommand(['target'])
        self.assertTrue(stdout.find('testtarget') != -1)
        stdout = self.runCheckCommand(['target', 'x86-linux-native'])
        if os.name == 'posix':
            # check that the settings file was created with the right permissions
            self.assertFalse(
                os.stat(os.path.join(self.test_dir, '.yotta.json')).st_mode & Check_Not_Stat
            )

    def test_setNonexistentTarget(self):
        stdout, stderr, statuscode = cli.run(['target', 'thisdoesnotexist'], cwd=self.test_dir)
        self.assertNotEqual(statuscode, 0)
        self.assertNotIn('Exception', stdout+stderr)
        self.assertIn('does not exist in the targets registry', stdout+stderr)

    def runCheckCommand(self, args):
        stdout, stderr, statuscode = cli.run(args, cwd=self.test_dir)
        self.assertEqual(statuscode, 0)
        return stdout or stderr


if __name__ == '__main__':
    unittest.main()





#!/usr/bin/env python
# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import unittest
import copy

# internal modules:
from yotta.test.cli import cli
from yotta.test.cli import util

Test_Min_Version_Insufficient = copy.copy(util.Test_Trivial_Lib)
Test_Min_Version_Insufficient['module.json'] = '''{
  "name": "test-trivial-lib",
  "version": "1.0.0",
  "description": "Module to test trivial lib compilation",
  "license": "Apache-2.0",
  "dependencies": {
  },
  "yotta": ">9999.9999.9999"
}'''

Test_Min_Version_OK = copy.copy(util.Test_Trivial_Lib)
Test_Min_Version_OK['module.json'] = '''{
  "name": "test-trivial-lib",
  "version": "1.0.0",
  "description": "Module to test trivial lib compilation",
  "license": "Apache-2.0",
  "dependencies": {
  },
  "yotta": ">0.0.1 !0.0.2 <999.999.999"
}'''

# NB: can't use this target to build
Test_Min_Version_Target_Name = 'test-target'
Test_Min_Version_Target_Insufficient = copy.copy(util.Test_Trivial_Lib)
Test_Min_Version_Target_Insufficient['yotta_targets/test-target/target.json'] = '''{
  "name": "%s",
  "version": "1.0.0",
  "license": "Apache-2.0",
  "yotta": ">9999.9999.9999"
}''' % Test_Min_Version_Target_Name

Test_Unparsable_Spec = copy.copy(util.Test_Trivial_Lib)
Test_Unparsable_Spec['module.json'] = '''{
  "name": "test-trivial-lib",
  "version": "1.0.0",
  "description": "Module to test trivial lib compilation",
  "license": "Apache-2.0",
  "dependencies": {
  },
  "yotta": "not something that yotta can parse as a version spec"
}'''
Test_Unparsable_Spec['yotta_targets/test-target/target.json'] = '''{
  "name": "%s",
  "version": "1.0.0",
  "license": "Apache-2.0",
  "yotta": ">9999.9999.9999"
}''' % Test_Min_Version_Target_Name


class TestCLIYottaVersionSpecs(unittest.TestCase):
    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_yottaVersionCheckTooLowBuilding(self):
        test_dir = util.writeTestFiles(Test_Min_Version_Insufficient)
        stdout, stderr, statuscode = cli.run(['--target', util.nativeTarget(), 'build'], cwd=test_dir)
        self.assertNotEqual(statuscode, 0)
        self.assertIn('requires yotta version >', stdout+stderr)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_yottaVersionCheckOKBuilding(self):
        test_dir = util.writeTestFiles(Test_Min_Version_OK)
        stdout, stderr, statuscode = cli.run(['--target', util.nativeTarget(), 'build'], cwd=test_dir)
        self.assertEqual(statuscode, 0)
        self.assertNotIn('requires yotta version >', stdout+stderr)
        util.rmRf(test_dir)

    def test_yottaVersionCheckTooLowInstalling(self):
        test_dir = util.writeTestFiles(Test_Min_Version_Insufficient)
        stdout, stderr, statuscode = cli.run(['--target', 'x86-linux-native', 'install'], cwd=test_dir)
        self.assertNotEqual(statuscode, 0)
        self.assertIn('requires yotta version >', stdout+stderr)
        util.rmRf(test_dir)

    def test_yottaVersionCheckOKInstalling(self):
        test_dir = util.writeTestFiles(Test_Min_Version_OK)
        stdout, stderr, statuscode = cli.run(['--target', 'x86-linux-native', 'install'], cwd=test_dir)
        self.assertEqual(statuscode, 0)
        self.assertNotIn('requires yotta version >', stdout+stderr)
        util.rmRf(test_dir)

    def test_yottaVersionTargetCheck(self):
        test_dir = util.writeTestFiles(Test_Min_Version_Target_Insufficient)
        stdout, stderr, statuscode = cli.run(['--target', Test_Min_Version_Target_Name, 'install'], cwd=test_dir)
        self.assertNotEqual(statuscode, 0)
        self.assertIn('requires yotta version >', stdout+stderr)
        util.rmRf(test_dir)

    def test_unparseableSpec(self):
        test_dir = util.writeTestFiles(Test_Unparsable_Spec)
        stdout, stderr, statuscode = cli.run(['--target', Test_Min_Version_Target_Name, 'install'], cwd=test_dir)
        self.assertNotEqual(statuscode, 0)
        self.assertIn('could not parse yotta version spec', stdout+stderr)
        util.rmRf(test_dir)





# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
from __future__ import print_function
import itertools

# Registry Access, , access packages in the registry, internal
from yotta.lib import registry_access
from yotta.lib import settings
from yotta.lib import version

def addOptions(parser):
    parser.add_argument(
        'type', choices=['module', 'target', 'both'], nargs='?', default='both',
        help="What to search (software modules, build targets, or both)"
    )
    parser.add_argument(
        'query', type=str, nargs=1, default='',
        help='Search query text. All items that contain the specified words '+
             'in their description, name, or keywords will be returned.'
    )
    parser.add_argument(
        '-k', '--keyword', default=[], action='append', dest='kw',
        help='Additional keywords to constrain the search. All specified '+
             'keywords must exist in an item\'s keywords list for it to '+
             'be returned.'
    )
    parser.add_argument(
        '-l', '--limit', default=10, type=int,
        help='Limit the number of results returned.'
    )
    parser.add_argument(
        '-s', '--short', default=False, action='store_true',
        help='Use short output format.'
    )

def lengthLimit(s, l):
    if len(s) > l:
        return s[:l-3] + '...'
    return s

def formatResult(result, plain=False, short=False, indent=''):
    if not plain:
        # colorama, BSD 3-Clause license, cross-platform terminal colours, pip install colorama
        import colorama
        DIM    = colorama.Style.DIM       #pylint: disable=no-member
        BRIGHT = colorama.Style.BRIGHT    #pylint: disable=no-member
        NORMAL = colorama.Style.NORMAL    #pylint: disable=no-member
        GREEN  = colorama.Fore.GREEN      #pylint: disable=no-member
        BLUE   = colorama.Fore.BLUE       #pylint: disable=no-member
        RED    = colorama.Fore.RED        #pylint: disable=no-member
        YELLOW = colorama.Fore.YELLOW     #pylint: disable=no-member
        RESET  = colorama.Style.RESET_ALL #pylint: disable=no-member
    else:
        DIM = BRIGHT = NORMAL = GREEN = BLUE = RED = YELLOW = RESET = u''

    def formatKeyword(keyword):
        if keyword.endswith('official'):
            return BRIGHT + GREEN + keyword + RESET
        else:
            return BRIGHT + keyword + NORMAL

    def formatAuthor(author):
        if isinstance(author, dict):
            return author.get('name', '<name unknown>') + ' ' + author.get('email', '<email unknown')
        else:
            return author

    # --short:
    # module-name module-version: description description

    # not --short:
    # module-name module-version
    #    description description description
    #    keyword, keyword, keyword
    #    author, author

    v = version.Version(result['version'])
    if v.major() < 1:
        if v.minor() < 1:
            ver_string = RED + result['version'] + RESET
        else:
            ver_string = YELLOW + result['version'] + RESET
    else:
        ver_string = GREEN + result['version'] + RESET

    name_string = BRIGHT+result['name']+NORMAL

    if 'description' in result:
        description = result['description']
        description_colour = ''
    else:
        description = '<description missing>'
        description_colour = RED

    if short:
        description = description_colour + lengthLimit(description, 160).replace('\n', ' ') + RESET
    else:
        description = description_colour + lengthLimit(description, 800).replace('\n', indent+' ').rstrip('\n') + RESET

    if short:
        return indent+u'%s %s: %s%s' % (name_string, ver_string, description, RESET)
    else:
        r = indent+(u'%s %s:\n' % (name_string, ver_string)) +\
            indent+(u'    %s\n' % (description))
        if 'keywords' in result and result['keywords']:
            r += indent+(u'    %s\n' % (', '.join([formatKeyword(k) for k in result['keywords']])))

        if 'author' in result and result['author']:
            authors = [result['author']]
        elif 'maintainers' in result and len(result['maintainers']):
            authors = result['maintainers']
        else:
            authors = []
        if len(authors):
            r += indent+(u'    %s\n' % (DIM + (', '.join([formatAuthor(a) for a in authors])) + RESET))

        return r.rstrip('\n')

def execCommand(args, following_args):
    success = False
    for result in itertools.islice(registry_access.search(query=args.query, keywords=args.kw, registry=args.registry), args.limit):
        success= True
        if args.type == 'both' or args.type == result['type']:
            print(formatResult(result, args.plain, short=args.short))
    for repo in filter(lambda s: 'type' in s and s['type'] == 'registry', settings.get('sources') or []) :
        count = 0
        print('')
        print('additional results from %s:' % repo['url'])
        for result in itertools.islice(registry_access.search(query=args.query, keywords=args.kw, registry=repo['url']), args.limit):
            success= True
            if args.type == 'both' or args.type == result['type']:
                print(formatResult(result, args.plain, indent='  ', short=args.short))
    # exit status: success if we found something, otherwise fail
    return 0 if success else 1




# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# workarounds for various issues in CMake

# standard library modules, , ,
import os
import logging


logger = logging.getLogger('yotta.lib.cmake_fixups')

def fixupEclipseProject(builddir, component):
    # explicitly add link to the source directory to generated eclipse
    # projects, so the source files are accessible through the GUI
    import fileinput
    import sys
    logger.debug('fixing up Eclipse project file')
    proj_file = os.path.join(builddir, '.project')
    done = False
    for line in fileinput.input(proj_file, inplace=1):
        sys.stdout.write(line)
        if line.strip() == '<linkedResources>' and not done:
            done = True
            # insert:
            #<link>
            #	<name>ComponentName-source</name>
            #	<type>2</type>
            #	<location>/absolute/path/to/source/location>
            #</link>
            # Note that the path must use forward slashes as separators on
            # windows.
            sys.stdout.write(
                '''\t\t<link>
\t\t\t<name>%s-source</name>
\t\t\t<type>2</type>
\t\t\t<location>%s</location>
\t\t</link>\n''' % (component.getName(), os.path.abspath(os.path.join(component.path, 'source')).replace('\\', '/'))
            )

def fixupNinjaBackslashes(builddir):
    logger.debug("Converting back-slashes in build.ninja to forward-slashes")
    build_file = os.path.join(builddir, "build.ninja")
    # We want to convert back-slashes to forward-slashes, except in macro
    # definitions, such as -DYOTTA_COMPONENT_VERSION = \"0.0.1\". So we use a
    # little trick: first we change all \" strings to an unprintable ASCII char
    # that can't appear in build.ninja (in this case \1), then we convert all
    # the back-slashed to forward-slashes, then we convert '\1' back to \".
    try:
        f = open(build_file, "r+t")
        data = f.read()
        data = data.replace('\\"', '\1')
        data = data.replace('\\', '/')
        data = data.replace('\1', '\\"')
        f.seek(0)
        f.write(data)
        f.close()
    except Exception as e:
        logger.error('Unable to fixup ninja file "%s", the build may fail.',
                build_file)


def applyFixupsForFenerator(cmake_generator, builddir, component):
    # cmake error: the generated Ninja build file will not work on windows when
    # arguments are read from a file (@file) instead of the command line, since
    # '\' in @file is interpreted as an escape sequence.
    #!!! FIXME: remove this once http://www.cmake.org/Bug/view.php?id=15278 is fixed!
    if ('Ninja' in cmake_generator) and os.name == 'nt':
        fixupNinjaBackslashes(builddir)

    # both fixups may apply, so don't use elif
    if "Eclipse" in cmake_generator:
        fixupEclipseProject(builddir, component)



# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import argparse

# Registry Access, , access modules in the registry, internal
from yotta.lib import registry_access

def addOptions(parser):
    # the registry URL to provide login info for
    parser.add_argument('--registry', dest='registry', help=argparse.SUPPRESS)

def execCommand(args, following_args):
    email_address = registry_access.whoami(args.registry)
    if email_address is not None:
        print(email_address)
        return 0
    else:
        print('not logged in')
        return 1





# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library options
from argparse import Action, SUPPRESS


class RegistryAction(Action):
    def __init__(self, *args, **kwargs):
        kwargs['nargs'] = 1
        self.dest = kwargs['dest']
        super(RegistryAction, self).__init__(*args, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values[0])

def addTo(parser):
    parser.add_argument(
        '--registry', default=None, dest='registry', help=SUPPRESS,
        action=RegistryAction
    )



# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library options
from argparse import Action, SUPPRESS
import logging

# logging setup, , setup the logging system, internal
from yotta.lib import logging_setup

def logLevelFromVerbosity(v):
    if v >= 4:
        return logging.NOTSET
    elif v == 3:
        return logging.DEBUG
    elif v == 2:
        return logging.DEBUG+3
    elif v == 1:
        return logging.DEBUG+7
    else:
        return logging.INFO

class VerbosityAction(Action):
    def __init__(self, *args, **kwargs):
        self.level = 0
        kwargs['nargs'] = 0
        kwargs['dest'] = SUPPRESS
        super(VerbosityAction, self).__init__(*args, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        self.level += 1
        loglevel = logLevelFromVerbosity(self.level)
        logging_setup.setLevel(loglevel)

def addTo(parser):
    parser.add_argument('-v', '--verbose', action=VerbosityAction,
        default=0,
        help='increase verbosity: can be used multiple times'
    )



#!/usr/bin/env python
# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest
import os
import tempfile

# internal modules:
from yotta.lib.fsutils import rmRf
from yotta.test.cli import cli
from yotta.test.cli import util

Test_Target = "x86-osx-native,*"

Private_Module_JSON = '''{
  "name": "testmod",
  "version": "0.0.0",
  "description": "a test module",
  "private": true,
  "keywords": [
  ],
  "author": "Someone Somewhere <someone.somewhere@yottabuild.org>",
  "repository": {
    "url": "git@github.com:mbedmicro/testmod.git",
    "type": "git"
  },
  "homepage": "https://github.com/mbedmicro/testmod",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "dependencies": {
    "testing-dummy": "*"
  },
  "targetDependencies": {
    "x86-osx-native": {
        "other-testing-dummy": "git@bitbucket.org:autopulated/other-testing-dummy.git#0.0.1"
    }
  }
}
'''

Public_Module_JSON = '''{
  "name": "testmod",
  "version": "0.0.0",
  "license": "Apache-2.0"
}'''


Test_Publish = {
'module.json':'''{
  "name": "test-publish",
  "version": "0.0.0",
  "description": "Test yotta publish",
  "author": "James Crosby <james.crosby@arm.com>",
  "license": "Apache-2.0",
  "keywords": ["mbed-official"],
  "dependencies":{
  }
}''',
'readme.md':'''##This is a test module used in yotta's test suite.''',
'source/foo.c':'''#include "stdio.h"
int foo(){
    printf("foo!\\n");
    return 7;
}'''
}

Test_prePublish_Prevents_Publish = {
  'module.json':'''{
  "name": "test-publish-prevented",
  "version": "0.0.0",
  "description": "Test yotta publish",
  "author": "James Crosby <james.crosby@arm.com>",
  "license": "Apache-2.0",
  "scripts": {
    "prePublish": "false"
  }
}''',
'readme.md':'''##This is a test module used in yotta's test suite.''',
}


class TestCLIPublish(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.test_dir = tempfile.mkdtemp()

    @classmethod
    def tearDownClass(cls):
        rmRf(cls.test_dir)

    def test_publishPrivate(self):
        with open(os.path.join(self.test_dir, 'module.json'), 'w') as f:
            f.write(Private_Module_JSON)
        stdout, stderr, status = cli.run(['--target', Test_Target, 'publish'], cwd=self.test_dir)
        self.assertNotEqual(status, 0)
        self.assertTrue('is private and cannot be published' in ('%s %s' % (stdout, stderr)))

    def test_publishNotAuthed(self):
        # ensure we're not logged in by setting a different settings directory:
        saved_settings_dir = None
        if 'YOTTA_USER_SETTINGS_DIR' in os.environ:
            saved_settings_dir = os.environ['YOTTA_USER_SETTINGS_DIR']
        os.environ['YOTTA_USER_SETTINGS_DIR'] = 'tmp_yotta_settings'
        try:
            with open(os.path.join(self.test_dir, 'module.json'), 'w') as f:
                f.write(Public_Module_JSON)
            stdout, stderr, status = cli.run(['-n', '--target', Test_Target, 'publish'], cwd=self.test_dir)
            if status != 0:
                out = stdout+stderr
                self.assertTrue(out.find('login required') != -1 or out.find('not module owner') != -1)
        finally:
            if saved_settings_dir is not None:
                os.environ['YOTTA_USER_SETTINGS_DIR'] = saved_settings_dir
            else:
                del os.environ['YOTTA_USER_SETTINGS_DIR']

    def test_prePublishPreventsPublish(self):
        path = util.writeTestFiles(Test_prePublish_Prevents_Publish, True)

        stdout, stderr, statuscode = cli.run(['-t', 'x86-linux-native', '--noninteractive', 'publish'], cwd=path)
        self.assertNotEqual(statuscode, 0)
        self.assertIn('prePublish script error code 1 prevents publishing', stdout + stderr)

        util.rmRf(path)

    def test_warnOfficialKeywords(self):
        path = util.writeTestFiles(Test_Publish, True)

        stdout, stderr, statuscode = cli.run(['-t', 'x86-linux-native', '--noninteractive', 'publish'], cwd=path)
        self.assertNotEqual(statuscode, 0)
        self.assertIn('Is this really an officially supported mbed module', stdout + stderr)

        util.rmRf(path)

if __name__ == '__main__':
    unittest.main()







# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# Utilities for parsing version source URLs, and determining how the
# specified version should be fetched.

# standard library modules, , ,
try:
    from urlparse import urlsplit
except ImportError:
    from urllib.parse import urlsplit #pylint: disable=no-name-in-module,import-error

# version, , represent versions and specifications, internal
from yotta.lib import version

class VersionSource(object):
    def __init__(self, source_type, location, spec):
        assert(source_type in ('registry', 'github', 'git', 'hg'))
        self.source_type = source_type
        self.location = location
        self.spec = spec
        try:
            self.semantic_spec = version.Spec(spec)
        except ValueError:
            # for git/github source URLs the spec is allowed to be a branch
            # name or tag name, as well as a valid semantic version
            # specification
            # !!! TODO: also allow hg here
            if source_type in ('git', 'github'):
                self.semantic_spec = None
            else:
                raise ValueError(
                    "Invalid semantic version spec: \"%s\"" % spec
                )

    def semanticSpec(self):
        return self.semantic_spec or version.Spec('*')

    def semanticSpecMatches(self, v):
        if isinstance(v, str):
            v = version.Version(v)
        if self.semantic_spec is None:
            return True
        else:
            return self.semantic_spec.match(v)


def _splitFragment(url):
    parsed = urlsplit(url)
    if '#' in url:
        return url[:url.index('#')], parsed.fragment
    else:
        return url, None

def _getGithubRef(source_url):
    import re
    # something/something#spec = github
    defragmented, fragment = _splitFragment(source_url)
    github_match = re.match('^[a-z0-9_-]+/([a-z0-9_-]+)$', defragmented, re.I)
    if github_match:
        return github_match.group(1), VersionSource('github', defragmented, fragment)

    # something/something@spec = github
    alternate_github_match = re.match('([a-z0-9_-]+/([a-z0-9_-]+)) *@?([~^><=.0-9a-z\*-]*)$', source_url, re.I)
    if alternate_github_match:
        return alternate_github_match.group(2), VersionSource('github', alternate_github_match.group(1), alternate_github_match.group(3))

    return None, None

def parseSourceURL(source_url):
    ''' Parse the specified version source URL (or version spec), and return an
        instance of VersionSource
    '''
    import re
    parsed = urlsplit(source_url)

    if '#' in source_url:
        without_fragment = source_url[:source_url.index('#')]
    else:
        without_fragment = source_url

    try:
        url_is_spec = version.Spec(source_url)
    except ValueError:
        url_is_spec = None

    if url_is_spec is not None:
        # if the url is an unadorned version specification (including an empty
        # string) then the source is the module registry:
        return VersionSource('registry', '', source_url)
    elif parsed.netloc.endswith('github.com'):
        # any URL onto github should be fetched over the github API, even if it
        # would parse as a valid git URL
        return VersionSource('github', parsed.path, parsed.fragment)
    elif parsed.scheme.startswith('git+') or parsed.path.endswith('.git'):
        # git+anything://anything or anything.git is a git repo:
        return VersionSource('git', without_fragment, parsed.fragment)
    elif parsed.scheme.startswith('hg+') or parsed.path.endswith('.hg'):
        # hg+anything://anything or anything.hg is a hg repo:
        return VersionSource('hg', without_fragment, parsed.fragment)

    # something/something@spec = github
    # something/something#spec = github
    module_name, github_match = _getGithubRef(source_url)
    if github_match:
        return github_match

    raise ValueError("Invalid version specification: \"%s\"" % (source_url))

def parseModuleNameAndSpec(module_name_and_spec):
    ''' Parse modulename[@versionspec] and return a tuple
        (module_name_string, version_spec_string).

        Also accepts raw github version specs (Owner/reponame#whatever), as the
        name can be deduced from these.

        Note that the specification split from the name is not validated. If
        there is no specification (just a module name) passed in, then '*' will
        be returned as the specification.
    '''
    import re
    # fist check if this is a raw github specification that we can get the
    # module name from:
    name, spec = _getGithubRef(module_name_and_spec)
    if name and spec:
        return name, module_name_and_spec

    # next split at the first @, if any
    name = module_name_and_spec.split('@')[0]
    spec = module_name_and_spec[len(name)+1:]

    name = name.strip()

    # if there's no specification, return the explicit any-version
    # specification:
    if not spec:
        spec = '*'

    import sys
    sys.stdout.flush()

    return name, spec



#!/usr/bin/env python
# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import unittest
import copy
import time
import tempfile

# internal modules:
from yotta.lib.detect import systemDefaultTarget
from yotta.test.cli import cli
from yotta.test.cli import util

Test_Tests = {
'module.json':'''{
  "name": "test-tests",
  "version": "0.0.0",
  "description": "Test yotta's compilation of tests.",
  "author": "James Crosby <james.crosby@arm.com>",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ]
}''',
'source/foo.c':'''#include "stdio.h"
int foo(){
    printf("foo!\\n");
    return 7;
}''',
'test-tests/foo.h':'int foo();',
'test/a/bar.c':'#include "test-tests/foo.h"\nint main(){ foo(); return 0; }',
'test/b/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/b/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/c/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/c/b/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/d/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/d/a/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/e/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/e/b/a/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/f/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/f/a/b/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/g/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/g/a/a/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }'
}

Test_Fitler_Pass = copy.copy(Test_Tests)
Test_Fitler_Pass['module.json'] = '''{
  "name": "test-tests",
  "version": "0.0.0",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "scripts": {
    "testReporter": [
      "grep",
      "!"
    ]
  }
}'''

Test_Fitler_Fail = copy.copy(Test_Tests)
Test_Fitler_Fail['module.json'] = '''{
  "name": "test-tests",
  "version": "0.0.0",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "scripts": {
    "testReporter": [
      "grep",
      "string that isnt in the output"
    ]
  }
}'''

Test_Fitler_NotFound = copy.copy(Test_Tests)
Test_Fitler_NotFound['module.json'] = '''{
  "name": "test-tests",
  "version": "0.0.0",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "scripts": {
    "testReporter": [
      "commandthatshouldntexist"
    ]
  }
}'''

Test_Reporter_Script = '''
#! python

import threading
import sys
try:
    import Queue as queue
except ImportError:
    # queue module has a different name in python 3 :(
    import queue

args = sys.argv[1:]

def tryFloat(x):
    try:
        return float(x)
    except:
        return None

Verbose = '--verbose' in args
Delay = sum([x for x in [tryFloat(x) for x in args] if x is not None])

exitcode = 1

def consumeNonBlocking():
    global exitcode
    all_output = ''
    buf = sys.stdin.read(1)
    while buf:
        all_output += buf
        if Verbose:
            print('[[ >>>', buf, ']]')
        if '[pass]' in all_output:
            exitcode = 0
            sys.exit(0)
        if '[fail]' in all_output:
            exitcode = 1
            sys.exit(1)
        buf = sys.stdin.read(1)

if Verbose:
    print('[[starting consume thread]]')
t = threading.Thread(target=consumeNonBlocking)
t.daemon = True
t.start()

if Verbose:
    print('[[thread was started]]')

t.join(4.5)

if Verbose:
    print('[[thread was joined]]')
    print('[[doing some more stuff before process exit]]')
    print('[[' + 'verbose'*1000+ ']]')
    print('[[script args were:', sys.argv[1:], ']]')

if Delay:
    import time
    time.sleep(Delay)
sys.exit(exitcode)
'''

# matrix of test configurations:
#  * Test passes / fails
#  * Test is slow / fast
#  * Test reporter is present / not
#  * If the test reporter is present:
#       * the test does / doesn't print extra data after pass/fail
#       * the reporter does/doesn't wait after the script has exited
#       * the reporter does/doesn't print information to stdout

def forAllReporterTests(fn):
    for reporter in [True, False]:
        for test_passes in [True, False]:
            for test_speed in ['slow', 'fast']:
                for test_verbose in [True, False]:
                    if reporter:
                        for reporter_waits in [2, 0]:
                            for reporter_verbose in [True, False]:
                                fn(
                                           reporter=reporter,
                                        test_passes=test_passes,
                                         test_speed=test_speed,
                                       test_verbose=test_verbose,
                                     reporter_waits=reporter_waits,
                                   reporter_verbose=reporter_verbose
                               )
                    else:
                        fn(
                                   reporter=reporter,
                                test_passes=test_passes,
                                 test_speed=test_speed,
                               test_verbose=test_verbose,
                             reporter_waits=0,
                           reporter_verbose=None
                       )

def cFileForReporterTest(**kwargs):
    r = '#include <unistd.h>\n#include <stdio.h>\n\nint main(){\n'
    if kwargs['reporter']:
        # slow reporter tests delay after printing
        if kwargs['test_verbose']:
            r += '  printf("'+'testverbose0'*77+'");\n'
        r += '  printf("[pass]\\n\\r");\n' if kwargs['test_passes'] else ' printf("[fail]\\n\\r");\n'
        if kwargs['test_verbose']:
            r += '  printf("'+'testverbose1'*22+'");\n'
        if kwargs['test_speed'] == 'slow':
            r += '  sleep(2);\n'
        if kwargs['test_verbose']:
            r += '  printf("'+'testverbose2'*33+'");\n'
        r += '  return 1;\n'
    else:
        if kwargs['test_verbose']:
            r += '  printf("'+'testverbose0'*22+'");\n'
        # slow non-reporter tests delay before exiting
        if kwargs['test_speed'] == 'slow':
            r += '  sleep(2);\n'
            if kwargs['test_verbose']:
                r += '  printf("'+'testverbose1'*77+'");\n'
        r += ' return 0;\n' if kwargs['test_passes'] else '  return 17;\n'
    r += '}\n'
    return r



def filesForReporterTest(**kwargs):
    Test = {
    }
    if kwargs['reporter']:
        reporter_command = '["python", "./report.py"'
        if kwargs['reporter_verbose']:
            reporter_command += ', "--verbose"'
        if kwargs['reporter_waits']:
            reporter_command += (', "%s"' % kwargs['reporter_waits'])
        reporter_command += "]"
        Test['module.json'] = '{"name": "reporter", "version": "0.0.0", "license": "Apache-2.0", "scripts":{"testReporter":'+reporter_command+'}}\n'
        Test['report.py'] = Test_Reporter_Script
    else:
        Test['module.json'] = '{"name": "reporter", "version": "0.0.0", "license": "Apache-2.0"}\n'
    Test['test/x.c'] = cFileForReporterTest(**kwargs)
    return Test


class TestCLITest(unittest.TestCase):
    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_tests(self):
        test_dir = util.writeTestFiles(Test_Tests, True)
        output = self.runCheckCommand(['--target', systemDefaultTarget(), 'build'], test_dir)
        output = self.runCheckCommand(['--target', systemDefaultTarget(), 'test'], test_dir)
        self.assertIn('test-a passed', output)
        self.assertIn('test-c passed', output)
        self.assertIn('test-d passed', output)
        self.assertIn('test-e passed', output)
        self.assertIn('test-f passed', output)
        self.assertIn('test-g passed', output)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_testOutputFilterPassing(self):
        test_dir = util.writeTestFiles(Test_Fitler_Pass, True)
        stdout = self.runCheckCommand(['--target', systemDefaultTarget(), 'test'], test_dir)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_testOutputFilterFailing(self):
        test_dir = util.writeTestFiles(Test_Fitler_Fail, True)
        stdout, stderr, statuscode = cli.run(['--target', systemDefaultTarget(), 'test'], cwd=test_dir)
        if statuscode == 0:
            print(stdout)
            print(stderr)
        self.assertIn('test-a failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-c failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-d failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-e failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-f failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-g failed', '%s %s' % (stdout, stderr))
        self.assertNotEqual(statuscode, 0)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_testOutputFilterNotFound(self):
        test_dir = util.writeTestFiles(Test_Fitler_NotFound, True)
        stdout, stderr, statuscode = cli.run(['--target', systemDefaultTarget(), 'test'], cwd=test_dir)
        if statuscode == 0:
            print(stdout)
            print(stderr)
        self.assertNotEqual(statuscode, 0)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_testCustomCMake(self):
        test_dir = util.writeTestFiles(util.Test_Test_Custom_CMake, True)
        output = self.runCheckCommand(['--target', systemDefaultTarget(), 'test'], test_dir)
        self.assertIn('test-trivial-lib-maintest passed', output)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_testAdditionalCMake(self):
        test_dir = util.writeTestFiles(util.Test_Test_Extra_CMake, True)
        output = self.runCheckCommand(['--target', systemDefaultTarget(), 'test'], test_dir)
        self.assertIn('test-trivial-lib-test-main passed', output)
        util.rmRf(test_dir)

    def runCheckCommand(self, args, test_dir):
        stdout, stderr, statuscode = cli.run(args, cwd=test_dir)
        if statuscode != 0:
            print('command failed with status %s' % statuscode)
            print(stdout)
            print(stderr)
        self.assertEqual(statuscode, 0)
        return '%s %s' % (stdout, stderr)

# the generated tests share a single test directory, so that the target
# descriptions etc. don't have to be re-downloaded many times (this makes them
# much faster)
class TestCLITestGenerated(TestCLITest):
    @classmethod
    def setUpClass(cls):
        cls.test_dir = tempfile.mkdtemp() + 'spaces in path'

    @classmethod
    def tearDownClass(cls):
        util.rmRf(cls.test_dir)

# generate the filter-testing tests dynamically:
def generateTestMethod(**kwargs):
    def generatedTestMethod(self):
        test_dir = util.writeTestFiles(filesForReporterTest(**kwargs), test_dir=self.test_dir)

        # build first, to make test timing more accurate:
        stdout, stderr, statuscode = cli.run(['--target', systemDefaultTarget(), 'build'], cwd=test_dir)
        #print('build:', stdout)
        #print('build:', stderr)
        #print('build statuscode was:', statuscode)
        self.assertEqual(statuscode, 0)

        tstart = time.time()
        stdout, stderr, statuscode = cli.run(['-vvv', '--target', systemDefaultTarget(), 'test'], cwd=test_dir)
        duration = time.time() - tstart

        # useful output for debugging failed tests:
        if bool(statuscode) == bool(kwargs['test_passes']) or \
                duration >= 5.5 + kwargs['reporter_waits'] or \
                (kwargs['test_speed'] == 'fast' and (duration >= 1.5 + kwargs['reporter_waits'])):
            print(stdout + stderr)
            print(statuscode)
            print('duration:', duration)

        if kwargs['test_passes']:
            self.assertEqual(statuscode, 0)
        else:
            self.assertNotEqual(statuscode, 0)

        # **no** tests should cause a timeout (Which is set at 4.5 seconds in
        # the test reporter), + the wait-for duration (+ 1 second slack for
        # process startup etc.)
        self.assertTrue(duration < 5.5 + kwargs['reporter_waits'])

        # if a test isn't slow, then it should run in less than 1 seconds
        if kwargs['test_speed'] == 'fast':
            self.assertTrue(duration < 1.5 + kwargs['reporter_waits'])
    return generatedTestMethod

def generateTest(**kwargs):
    test_name = "test_" + '_'.join([ '%s_%s' % (k, v) for k, v in sorted(kwargs.items(), key=lambda x: x[0])])
    test_method = generateTestMethod(**kwargs)
    test_method.__name__ = test_name
    setattr(TestCLITestGenerated, test_name, test_method)

if util.canBuildNatively():
    forAllReporterTests(generateTest)
else:
    print('WARNING: skipping test reporter tests (cannot build natively on this platform)')



#!/usr/bin/env python
# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import unittest
import subprocess
import copy
import re
import datetime

# internal modules:
from yotta.test.cli import cli
from yotta.test.cli import util

Test_Complex = {
'module.json': '''{
  "name": "test-testdep-a",
  "version": "0.0.2",
  "description": "Module to test test-dependencies",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "dependencies": {
    "test-testdep-b": "*",
    "test-testdep-c": "*",
    "test-testdep-d": "*"
  },
  "testDependencies": {
    "test-testdep-e": "*"
  }
}
''',

'source/a.c': '''
#include "a/a.h"
#include "b/b.h"
#include "c/c.h"
#include "d/d.h"

int a(){
    return 1 + b() + c() + d(); // 35
}
''',

'source/a.c': '''
#include "a/a.h"
#include "b/b.h"
#include "c/c.h"
#include "d/d.h"

int a(){
    return 1 + b() + c() + d(); // 35
}
''',

'a/a.h':'''
#ifndef __A_H__
#define __A_H__
int a();
#endif
''',

'test/check.c': '''
#include <stdio.h>

#include "a/a.h"
#include "b/b.h"
#include "c/c.h"
#include "d/d.h"
#include "e/e.h"


int main(){
    int result = a() + b() + c() + d() + e();
    printf("%d\\n", result);
    return !(result == 86);
}
'''
}

Test_Build_Info = copy.copy(util.Test_Trivial_Exe)
Test_Build_Info['source/lib.c'] = '''
#include "stdio.h"
#include YOTTA_BUILD_INFO_HEADER

#define STRINGIFY(s) STRINGIFY_INDIRECT(s)
#define STRINGIFY_INDIRECT(s) #s

int main(){
    printf("vcs ID: %s\\n", STRINGIFY(YOTTA_BUILD_VCS_ID));
    printf("vcs clean: %d\\n", YOTTA_BUILD_VCS_CLEAN);
    printf("build UUID: %s\\n", STRINGIFY(YOTTA_BUILD_UUID));
    printf(
        "build timestamp: %.4d-%.2d-%.2d-%.2d-%.2d-%.2d\\n",
        YOTTA_BUILD_YEAR,
        YOTTA_BUILD_MONTH,
        YOTTA_BUILD_DAY,
        YOTTA_BUILD_HOUR,
        YOTTA_BUILD_MINUTE,
        YOTTA_BUILD_SECOND
    );
    return 0;
}

'''

Test_Tests = {
'module.json':'''{
  "name": "test-tests",
  "version": "0.0.0",
  "description": "Test yotta's compilation of tests.",
  "keywords": [],
  "author": "James Crosby <james.crosby@arm.com>",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "dependencies": {},
  "targetDependencies": {}
}''',
'source/foo.c':'''#include "stdio.h"
int foo(){
    printf("foo!\\n");
    return 7;
}''',
'test-tests/foo.h':'int foo();',
'test/a/bar.c':'#include "test-tests/foo.h"\nint main(){ foo(); return 0; }',
'test/b/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/b/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/c/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/c/b/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/d/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/d/a/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/e/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/e/b/a/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/f/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/f/a/b/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/g/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/g/a/a/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }'
}

Test_Ignore_Custom_Cmake = {
'module.json':'''{
  "name": "cmake-ignore-test",
  "version": "0.0.0",
  "keywords": [],
  "author": "",
  "license": "Apache-2.0",
  "dependencies": {}
}''',
'.yotta_ignore':'''
./source/CMakeLists.txt
CMakeLists.txt
./source/empty.c
./source/ignoreme.cmake
./test/CMakeLists.txt
./test/ignoreme.cmake
''',
'source/CMakeLists.txt':'''
message("source CMakeLists.txt should be ignored!")
add_library(cmake-ignore-test "empty.c")
''',
'CMakeLists.txt': '''
message("root cmakelist should be ignored!")
add_subdirectory(source)
add_subdirectory(test)
''',
'source/empty.c': '''
int foo(){ }
''',
'source/ignoreme.cmake': '''
message(".cmake file should be ignored!")
''',
'test/CMakeLists.txt': '''
message("test CMakeLists.txt file should be ignored!")
''',
'test/ignoreme.cmake': '''
message("test .cmake file should be ignored!")
''',
'test/empty.c': '''
int main(){ return 0; }
'''
}

def _generateTestCustomLibBinDir(subdir='mysubdir', bin=False):
    if bin:
        libbin = 'bin'
        fnname = 'main'
    else:
        libbin = 'lib'
        fnname = 'foo'
    return {
        'module.json':'''{
          "name": "test-custom-dir-%s",
          "version": "1.0.0",
          "description": "Module to test trivial %s compilation",
          "license": "Apache-2.0",
          "%s": "%s"
        }''' % (libbin, libbin, libbin, subdir),
        'test-custom-dir-%s/%s.h' % (libbin, libbin): '''
        int %s();
        ''' % (fnname),
        '%s/%s.c' % (subdir, libbin):'''
        #warning "this message should be printed"
        #include "test-custom-dir-%s/%s.h"
        int %s(){ return 7; }
        ''' % (libbin, libbin, fnname)
    }

Test_Custom_Lib_Dir = _generateTestCustomLibBinDir('mylibdir', 'lib')
Test_Custom_Lib_Sub_Dir = _generateTestCustomLibBinDir('mylibdir/subdir', 'lib')
Test_Custom_Lib_Sub_Source_Dir = _generateTestCustomLibBinDir('source/mysourcesubdir', 'lib')
Test_Custom_Bin_Dir = _generateTestCustomLibBinDir('mybindir', 'bin')
Test_Custom_Bin_Sub_Dir = _generateTestCustomLibBinDir('mybindir/subdir', 'bin')
Test_Custom_Bin_Sub_Source_Dir = _generateTestCustomLibBinDir('source/somewhere/mybindir', 'bin')

# expect an error message
Test_Lib_And_Bin = {
'module.json':'''{
  "name": "test-custom-dir-lib",
  "version": "1.0.0",
  "description": "Module to test trivial lib compilation",
  "license": "Apache-2.0",
  "lib": "mylibdir",
  "bin": "mylibdir"
}''',
'mylibdir/lib.c':'''
int foo(){ return 7; }
'''
}

# expect an error message
Test_Lib_Nonexistent = {
'module.json':'''{
  "name": "test-custom-dir-lib",
  "version": "1.0.0",
  "description": "Module to test trivial lib compilation",
  "license": "Apache-2.0",
  "lib": "doesntexist"
}'''
}

# expect an error message
Test_Bin_Nonexistent = {
'module.json':'''{
  "name": "test-custom-dir-lib",
  "version": "1.0.0",
  "description": "Module to test trivial lib compilation",
  "license": "Apache-2.0",
  "bin": "doesntexist"
}'''
}

# expect a warning message for "Source" and "src" implicit source directory
# names: only "source" is supported as an implicit source directory name:
Test_Misspelt_Source_Dir_1 = {
'module.json':'''{
  "name": "test-mod",
  "version": "1.0.0",
  "description": "Module to test misspelt source dir names",
  "license": "Apache-2.0"
}''',
'Source/lib.c':"int foo(){return 0;}\n"
}

Test_Misspelt_Source_Dir_2 = {
'module.json':'''{
  "name": "test-mod",
  "version": "1.0.0",
  "description": "Module to test misspelt source dir names",
  "license": "Apache-2.0"
}''',
'src/lib.c':"int foo(){return 0;}\n"
}

# ...but if the misspelt directory is ignored, then no warning should be issued
Test_Ignored_Misspelt_Source_Dir = {
'module.json':'''{
  "name": "test-mod",
  "version": "1.0.0",
  "description": "Module to test misspelt source dir names",
  "license": "Apache-2.0"
}''',
'src/lib.c':"int foo(){return 0;}\n",
'.yotta_ignore':"./src"
}


class TestCLIBuild(unittest.TestCase):
    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_buildTrivialLib(self):
        test_dir = util.writeTestFiles(util.Test_Trivial_Lib)

        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)

        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_buildTrivialExe(self):
        test_dir = util.writeTestFiles(util.Test_Trivial_Exe)

        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)

        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_buildComplex(self):
        test_dir = util.writeTestFiles(Test_Complex)

        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)

        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_buildComplexSpaceInPath(self):
        test_dir = util.writeTestFiles(Test_Complex, True)

        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)

        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_buildTests(self):
        test_dir = util.writeTestFiles(Test_Tests, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'test'], test_dir)
        self.assertIn('test-a', stdout)
        self.assertIn('test-c', stdout)
        self.assertIn('test-d', stdout)
        self.assertIn('test-e', stdout)
        self.assertIn('test-f', stdout)
        self.assertIn('test-g', stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_buildInfo(self):
        test_dir = util.writeTestFiles(Test_Build_Info, True)
        # commit all the test files to git so that the VCS build info gets
        # defined:
        # (set up the git user env vars so we can run git commit without barfing)
        util.setupGitUser()
        subprocess.check_call(['git', 'init', '-q'], cwd=test_dir)
        subprocess.check_call(['git', 'add', '.'], cwd=test_dir)
        subprocess.check_call(['git', 'commit', '-m', 'test build info automated commit', '-q'], cwd=test_dir)

        self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)

        build_time = datetime.datetime.utcnow()
        output = subprocess.check_output(['./build/' + util.nativeTarget().split(',')[0] + '/source/test-trivial-exe'], cwd=test_dir).decode()
        self.assertIn('vcs clean: 1', output)

        # check build timestamp
        self.assertIn('build timestamp: ', output)
        build_timestamp_s = re.search('build timestamp: (.*)\n', output)
        self.assertTrue(build_timestamp_s)
        build_timestamp_s = build_timestamp_s.group(1)
        build_time_parsed = datetime.datetime.strptime(build_timestamp_s, '%Y-%m-%d-%H-%M-%S')
        build_time_skew = build_time_parsed - build_time
        self.assertTrue(abs(build_time_skew.total_seconds()) < 3)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_extraCMakeBuild(self):
        test_dir = util.writeTestFiles(util.Test_Extra_CMake_Lib, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_customCMakeBuild(self):
        test_dir = util.writeTestFiles(util.Test_Custom_CMake_Lib, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_extraCMakeBuildExe(self):
        test_dir = util.writeTestFiles(util.Test_Extra_CMake_Exe, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_customCMakeBuildExe(self):
        test_dir = util.writeTestFiles(util.Test_Custom_CMake_Exe, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_ignoreCustomCMake(self):
        test_dir = util.writeTestFiles(Test_Ignore_Custom_Cmake, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertNotIn('should be ignored', stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_customLibDir(self):
        test_dir = util.writeTestFiles(Test_Custom_Lib_Dir, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn('this message should be printed', stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_customLibSubDir(self):
        test_dir = util.writeTestFiles(Test_Custom_Lib_Sub_Dir, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn('this message should be printed', stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_customLibSubSourceDir(self):
        test_dir = util.writeTestFiles(Test_Custom_Lib_Sub_Source_Dir, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn('this message should be printed', stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_customBinDir(self):
        test_dir = util.writeTestFiles(Test_Custom_Bin_Dir, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn('this message should be printed', stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_customBinSubDir(self):
        test_dir = util.writeTestFiles(Test_Custom_Bin_Sub_Dir, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn('this message should be printed', stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_customBinSubSourceDir(self):
        test_dir = util.writeTestFiles(Test_Custom_Bin_Sub_Source_Dir, True)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn('this message should be printed', stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_libAndBinSpecified(self):
        test_dir = util.writeTestFiles(Test_Lib_And_Bin)
        stdout, stderr, statuscode = cli.run(['--target', util.nativeTarget(), 'build'], cwd=test_dir)
        self.assertNotEqual(statuscode, 0)
        self.assertIn('Both "lib" and "bin" are specified in module.json: only one is allowed', stdout+stderr)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_libNonExistent(self):
        test_dir = util.writeTestFiles(Test_Lib_Nonexistent)
        stdout, stderr, statuscode = cli.run(['--target', util.nativeTarget(), 'build'], cwd=test_dir)
        self.assertIn('directory "doesntexist" doesn\'t exist', stdout+stderr)
        # !!! FIXME: should this error be fatal?
        # self.assertNotEqual(statuscode, 0)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_binNonExistent(self):
        test_dir = util.writeTestFiles(Test_Bin_Nonexistent)
        stdout, stderr, statuscode = cli.run(['--target', util.nativeTarget(), 'build'], cwd=test_dir)
        self.assertIn('directory "doesntexist" doesn\'t exist', stdout+stderr)
        # !!! FIXME: should this error be fatal?
        # self.assertNotEqual(statuscode, 0)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_misspeltSourceDir1(self):
        test_dir = util.writeTestFiles(Test_Misspelt_Source_Dir_1)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn("has non-standard source directory name", stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_misspeltSourceDir2(self):
        test_dir = util.writeTestFiles(Test_Misspelt_Source_Dir_2)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertIn("has non-standard source directory name", stdout)
        util.rmRf(test_dir)

    @unittest.skipIf(not util.canBuildNatively(), "can't build natively on windows yet")
    def test_misspeltSourceDirIgnored(self):
        test_dir = util.writeTestFiles(Test_Ignored_Misspelt_Source_Dir)
        stdout = self.runCheckCommand(['--target', util.nativeTarget(), 'build'], test_dir)
        self.assertNotIn("has non-standard source directory name", stdout)
        util.rmRf(test_dir)

    def runCheckCommand(self, args, test_dir):
        stdout, stderr, statuscode = cli.run(args, cwd=test_dir)
        if statuscode != 0:
            print('command failed with status %s' % statuscode)
            print(stdout)
            print(stderr)
        self.assertEqual(statuscode, 0)
        return stdout + stderr



# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# utility function to indicate whether the current item is the last one in a
# generator
def islast(generator):
    ''' indicate whether the current item is the last one in a generator
    '''
    next_x = None
    first = True
    for x in generator:
        if not first:
            yield (next_x, False)
        next_x = x
        first = False
    if not first:
        yield (next_x, True)



# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import logging

# validate, , validate things, internal
from yotta.lib import validate

def addOptions(parser):
    pass

def execCommand(args, following_args):
    p = validate.currentDirectoryModuleOrTarget()
    if not p:
        return 1

    error = p.unpublish(args.registry)
    if error:
        logging.error(error)
        return 1

    logging.info('unpublished version: %s', p.getVersion())
    return 0



# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import logging

# validate, , validate things, internal
from yotta.lib import validate
# options, , shared options, internal
import yotta.options as options

def addOptions(parser):
    options.force.addTo(parser)

# python 2 + 3 compatibility
try:
    global input
    input = raw_input
except NameError:
    pass

def prePublishCheck(p, force=False, interactive=True):
    need_ok = False
    if p.description.get('bin', None) is not None:
        logging.warning(
            'This is an executable application, not a re-usable library module. Other modules will not be able to depend on it!'
        )
        need_ok = True

    official_keywords = [x for x in p.getKeywords() if x.endswith('-official')]
    if len(official_keywords):
        need_ok = True
        for k in official_keywords:
            prefix = k[:-len('-official')]
            logging.warning(
                ('You\'re publishing with the %s tag. Is this really an '+
                'officially supported %s module? If not, please remove the %s '+
                'tag from your %s file. If you are unsure, please ask on the '+
                'issue tracker.') % (
                    k, prefix, k, p.description_filename
                )
            )

    if need_ok and not interactive:
        logging.error('--noninteractive prevents user confirmation. Please re-run with --force')
        return 1

    if need_ok and not force:
        input("If you still want to publish, press [enter] to continue.")

    return 0

def execCommand(args, following_args):
    p = validate.currentDirectoryModuleOrTarget()
    if not p:
        return 1

    if not p.vcsIsClean():
        logging.error('The working directory is not clean. Commit before publishing!')
        return 1

    errcode = prePublishCheck(p, args.force, args.interactive)
    if errcode and not args.force:
        return errcode

    errcode = p.runScript('prePublish')
    if errcode:
        logging.error("prePublish script error code %s prevents publishing", errcode)
        return errcode

    error = p.publish(args.registry)
    if error:
        logging.error(error)
        return 1

    errcode = p.runScript('postPublish')
    if errcode:
        logging.warning("postPublish script exited with code %s", errcode)

    # tag the version published as 'latest'
    # !!! can't do this, as can't move tags in git?
    #p.commitVCS(tag='latest')
    logging.info('published latest version: %s', p.getVersion())
    return errcode
